/*  Spruce
 *  Copyright (C) 1999 Susixware
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "date.h"

static gint get_days_in_month (gint mon, gint year)
{
   switch (mon)
   {
      case 1: case 3: case 5: case 7: case 8: case 10: case 12:
         return 31;
      case 4: case 6: case 9: case 11:
         return 30;
      case 2:
         if ((year % 4 == 0 && year % 100 != 0) || year % 400 == 0)
            return 29;
         return 28;
      default:
         return 30;
   }
}

static gint get_weekday (gchar *str)
{
   g_return_val_if_fail (str != NULL, 0);

   if (strncmp (str, "Mon", 3) == 0) {
      return 0;
   } else if (strncmp (str, "Tue", 3) == 0) {
      return 1;
   } else if (strncmp (str, "Wed", 3) == 0) {
      return 2;
   } else if (strncmp (str, "Thu", 3) == 0) {
      return 3;
   } else if (strncmp (str, "Fri", 3) == 0) {
      return 4;
   } else if (strncmp (str, "Sat", 3) == 0) {
      return 5;
   } else if (strncmp (str, "Sun", 3) == 0) {
      return 6;
   }

   return 0;
}

static gint get_month (gchar *str)
{
    g_return_val_if_fail (str != NULL, 0);
    
    if (strncmp (str, "Jan", 3) == 0) {
        return 0;
    } else if (strncmp (str, "Feb", 3) == 0) {
        return 1;
    } else if (strncmp (str, "Mar", 3) == 0) {
        return 2;
    } else if (strncmp (str, "Apr", 3) == 0) {
        return 3;
    } else if (strncmp (str, "May", 3) == 0) {
        return 4;
    } else if (strncmp (str, "Jun", 3) == 0) {
        return 5;
    } else if (strncmp (str, "Jul", 3) == 0) {
        return 6;
    } else if (strncmp (str, "Aug", 3) == 0) {
        return 7;
    } else if (strncmp (str, "Sep", 3) == 0) {
        return 8;
    } else if (strncmp (str, "Oct", 3) == 0) {
        return 9;
    } else if (strncmp (str, "Nov", 3) == 0) {
        return 10;
    } else if (strncmp (str, "Dec", 3) == 0) {
        return 11;
    }
    
    return 0;
}

struct tm* parse_date (gchar *datestr)
{
    struct tm* newdate;
    gchar month[4], tzone[6];
    gdouble tz;
    
    g_return_val_if_fail (datestr != NULL, NULL);
    
    newdate = g_new (struct tm, 1);
    
    if (strchr (datestr, ',')) {
        newdate->tm_wday = get_weekday (datestr);
        datestr += 5;
    } else {
        newdate->tm_wday = 0;
    }
    
    sscanf (datestr, "%d %3s %d %d:%d:%d %5s", &newdate->tm_mday,
            month, &newdate->tm_year, &newdate->tm_hour, &newdate->tm_min,
            &newdate->tm_sec, tzone);
    newdate->tm_mon = get_month (month);
    if (newdate->tm_year > 1900) newdate->tm_year -= 1900;
    
    tz = atoi (tzone) / 100.0;
    
    newdate->tm_min -= (gint)(tz - (gint) tz) * 100;
    if (newdate->tm_min > 59) { newdate->tm_min -= 60; newdate->tm_hour++; }
    else if (newdate->tm_min < 0) { newdate->tm_min += 60; newdate->tm_hour--; }
    
    newdate->tm_hour -= (gint)tz;
    if (newdate->tm_hour > 23) { newdate->tm_hour -= 24; newdate->tm_mday++; }
    else if (newdate->tm_hour < 0) { newdate->tm_hour += 24; newdate->tm_mday++; }
    
    if (newdate->tm_mday > get_days_in_month (newdate->tm_mon, newdate->tm_year)) {
        newdate->tm_mday -= get_days_in_month (newdate->tm_mon, newdate->tm_year);
        newdate->tm_mon++;
    } else if (newdate->tm_mday < 0) {
        newdate->tm_mday += get_days_in_month (newdate->tm_mon, newdate->tm_year);
        newdate->tm_mon--;
    }
    
    if (newdate->tm_mon > 11) { newdate->tm_mon -= 12; newdate->tm_year++; }
    else if (newdate->tm_mon < 0) { newdate->tm_mon += 12; newdate->tm_year --; }
    
    newdate->tm_isdst = -1;
    newdate->tm_yday = 0;
    
    return newdate;
}

gchar* print_date (struct tm* date)
{
   gchar* newdate;

   g_return_val_if_fail (date != NULL, NULL);

   newdate = g_new (gchar, 100);

   strftime (newdate, 100, "%a, %d %b %Y %H:%M:%S %z", date);

   return newdate;
}
