/*  Spruce
 *  Copyright (C) 1999 Susixware
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "main.h"

static gchar spruce_ver[] = "0.5.6";

/* lets keep track of open windows... */
extern gint AboutOpen;
extern gint AccountsOpen;
extern gint AddrBookOpen;
extern gint ComposerOpen;
extern gint FiltersOpen;
extern gint MesgReaderOpen;
extern gint OptionsOpen;
extern gint PrintOpen;

/* generic globals */
extern gchar *spruceconf;                       /* spruce config file */
extern gchar *AddrSelected;                     /* addr selected by frmAddrBook */
extern gint first_time;
extern gint texticons;                          /* use text, icons or both? */
extern gint NumMailboxes;                       /* number of mailboxes (Inbox, Outbox, etc) */

/* Identity */
extern gchar *SenderAddr;                       /* user's email addr */
extern gchar *ReplyTo;                          /* user's Reply-To email addr */
extern gint use_rand_sig;                       /* use spruce's random signature? */
extern gint auto_attach_sig;                    /* auto-attach the signature? */
extern gchar *signaturefile;                    /* user's signature file */
/* Accounts */
extern gint NumAccounts;                        /* number of pop3 accounts we have */
extern accounts_t AccountsList[MAX_ACCOUNTS];
extern Server SmtpServer;                       /* smtp server */
/* Appearance */
extern gchar *mesg_body_font;
/* Misc */
extern gint save_outgoing;                      /* save outgoing messages? */
extern gint empty_on_exit;                      /* empty "Deleted items" on exit? */
extern gint save_sizes;				/* save window and column sizes on exit? */
extern gint cm_delay;                           /* check mail every XX minutes */
extern gint hdr_trim_level;                     /* 0 = none; 1 = partial; 2 = full */
/* PGP */
extern gchar *pgp_identity;                     /* PGP Identity */
extern gint use_gnupg;                          /* do we use GNUpg or Pgp50i? */
extern gint pgp_sign;                           /* pgpv -s ? */
extern gint encrypt_to_self;                    /* always encrypt to self? */
extern gint store_passphrase;                   /* store our passphrase? */
extern gchar *pgp_passphrase;                   /* pgp passphrase */

extern GList *mime_parts;

/* here are our window widgets... */
extern GtkWidget *frmAbout;
extern GtkWidget *frmAccounts;
extern GtkWidget *frmAddrBook;
extern GtkWidget *frmComposer;
extern GtkWidget *frmFilters;
extern GtkWidget *frmMain;
extern GtkWidget *frmMesgReader;
extern GtkWidget *frmOptions;
extern GtkWidget *frmPrint;
extern GtkWidget *frmStatus;


/* local to main.c */
static gint cMesg = -1;
static gint NumMesgs = 0;
static gint cMbox = -1;
static gboolean refreshing_boxes    = FALSE;
static gboolean refreshing_maillist = FALSE;
static gboolean refreshing_message  = FALSE;
static gint MbxNewOpen;
static GtkWidget *dlgNewMailbox;
static gchar *SaveFile;
static gint SaveAsOpen = 0;
static GtkWidget *frmMainSaveAs;
static gint fetching_mail = 0;
static GtkWidget *dlgCopyMove;
static gint CopyMoveOpen = 0;
static gint cm_operation = 0;
static GtkWidget *mnuMessage_menu;
static GtkWidget *mnuMailboxes_menu;
static gint sort_column = 0;                        /* column to sort */
static GtkSortType sort_type = GTK_SORT_ASCENDING;
static gint mime;
static gint mime_preferred_part;

int main(int argc, char *argv[])
{
   reset_globals();

   gtk_set_locale();
   gtk_init(&argc, &argv);
   /*spruce_init(argc, argv);*/

   if (!rc_read_defaults())
   {
      fprintf(stderr, "Ah...this is your first time running Spruce eh?\n");
      fprintf(stderr, "If this segfaults, please run 'spruce.inst'\n");
      fprintf(stderr, "Thanks.\n");
      first_run();
      frmMain = create_frmMain();  /* we needa create the main window so we can save defaults */
      rc_save_defaults();
      
      reset_globals();
      first_time = 1;
      rc_read_defaults();  /* seems to crash the Options dialog if we don't do this...hmmm */
	}

   spruce_init(argc, argv); /* read command line after spruce.conf so command line has precedence */

   frmMain = create_frmMain();
   read_accounts();

   read_sizes();	   /* we must read the sizes after frmMain has been created */

   filters_read();

   gtk_widget_show(frmMain);

   if (first_time)
	{
      frmOptions = create_frmOptions();
      gtk_widget_show(frmOptions);
      OptionsOpen = 1;
	}

   refresh_mailboxes(TOP);

   /* Check for new mail once every XX minutes. */
   if (cm_delay > 0)
      gtk_timeout_add(cm_delay * 60 * 1000, periodic_mail_check, 0);

   gtk_main();

   return 0;
}

GtkWidget* create_frmMain ()
{
  GtkWidget *frmMain;
  GtkWidget *table;
  GtkWidget *menubar;
  GtkWidget *mnuFile;
  GtkWidget *mnuFile_menu;
  GtkWidget *mnuNew_Message;
  GtkWidget *mnuSave_As;
  GtkWidget *mnuPrint;
  GtkWidget *separator1;
  GtkWidget *separator2;
  GtkWidget *separator3;
  GtkWidget *separator4;
  GtkWidget *separator5;
  GtkWidget *separator6;
  GtkWidget *separator7;
  GtkWidget *mnuCheck_Mail;
  GtkWidget *mnuExit;
  GtkWidget *mnuEdit;
  GtkWidget *mnuEdit_menu;
  GtkWidget *mnuAddress_Book;
  GtkWidget *mnuAccounts;
  GtkWidget *mnuFilters;
  GtkWidget *mnuOptions;
  GtkWidget *mnuMailboxes;
  /*GtkWidget *mnuMailboxes_menu;*/
  GtkWidget *mnuMailboxCreate;
  GtkWidget *mnuMailboxDelete;
  GtkWidget *mnuMessage;
  /*GtkWidget *mnuMessage_menu;*/
  GtkWidget *mnuMesgNext;
  GtkWidget *mnuMesgPrevious;
  GtkWidget *mnuMesgReply;
  GtkWidget *mnuMesgForward;
  GtkWidget *mnuMesgOpen;
  GtkWidget *mnuMesgDelete;
  GtkWidget *mnuMesgExpunge;
  GtkWidget *mnuCopy_to;
  GtkWidget *mnuMove_to;
  GtkWidget *mnuMime;
  GtkWidget *mnuMime_menu;
  GtkWidget *mnuMimeSave;
  GtkWidget *mnuHelp;
  GtkWidget *mnuHelp_menu;
  GtkWidget *mnuAbout;
  GtkWidget *mnuHowto;
  GtkWidget *hboxBody;
  GtkWidget *txtMessage;
  GtkWidget *vsbMessage;
  GtkWidget *scrlBoxes;
  GtkWidget *clistBoxes;
  GtkWidget *label11;
  GtkWidget *scrlMail;
  GtkWidget *clistMail;
  GtkWidget *lblMesgID;
  GtkWidget *lblSender;
  GtkWidget *lblSubject;
  GtkWidget *lblDate;
  GtkAccelGroup *accel_group;  
  GtkWidget *handlebox;
  GtkWidget *toolbar = NULL;
  GtkWidget *iconw;
  GtkWidget *cmdNew, *cmdSave, *cmdReply, *cmdFwd, *cmdAddrbk, *cmdDel;
  GtkWidget *cmdCheckMail, *cmdPrevious, *cmdNext, *cmdPrint;
  GdkBitmap *mask;
  GdkPixmap *icon;
  GtkWidget *scrlMime;
  GtkWidget *clistMime;
  GtkTargetEntry target[] = { {"STRING", 0, 1},
  			      {"text/plain", 0, 2}};

  frmMain = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (frmMain, "frmMain");
  gtk_object_set_data (GTK_OBJECT (frmMain), "frmMain", frmMain);
  gtk_widget_set_usize (frmMain, 640, 480);
  gtk_signal_connect_object (GTK_OBJECT (frmMain), "destroy",
                             GTK_SIGNAL_FUNC (on_frmMain_destroy),
                             GTK_OBJECT (frmMain));
  gtk_window_set_title (GTK_WINDOW (frmMain), _("Spruce"));
  gtk_window_set_policy (GTK_WINDOW (frmMain), TRUE, TRUE, FALSE);
  gtk_widget_realize(frmMain);
  
  accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW (frmMain), accel_group);

  table = gtk_table_new (23, 18, FALSE);
  gtk_widget_set_name (table, "table");
  gtk_object_set_data (GTK_OBJECT (frmMain), "table", table);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frmMain), table);

  menubar = gtk_menu_bar_new ();
  gtk_widget_set_name (menubar, "menubar");
  gtk_object_set_data (GTK_OBJECT (frmMain), "menubar", menubar);
  gtk_widget_show (menubar);
  gtk_table_attach (GTK_TABLE (table), menubar, 0, 18, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) 0, 0, 0);

  mnuFile = gtk_menu_item_new_with_label (_("File"));
  gtk_widget_set_name (mnuFile, "mnuFile");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuFile", mnuFile);
  gtk_widget_show (mnuFile);
  gtk_container_add (GTK_CONTAINER (menubar), mnuFile);

  mnuFile_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuFile_menu, "mnuFile_menu");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuFile_menu", mnuFile_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuFile), mnuFile_menu);

  mnuNew_Message = gtk_menu_item_new_with_label (_("New Message"));
  gtk_widget_set_name (mnuNew_Message, "mnuNew_Message");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuNew_Message", mnuNew_Message);
  gtk_widget_show (mnuNew_Message);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuNew_Message);
  gtk_signal_connect (GTK_OBJECT (mnuNew_Message), "activate",
                      GTK_SIGNAL_FUNC (on_mnuNew_Message_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuNew_Message, "activate", accel_group,
                              GDK_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);


  mnuSave_As = gtk_menu_item_new_with_label (_("Save As..."));
  gtk_widget_set_name (mnuSave_As, "mnuSave_As");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuSave_As", mnuSave_As);
  gtk_widget_show (mnuSave_As);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuSave_As);
  gtk_signal_connect (GTK_OBJECT (mnuSave_As), "activate",
                      GTK_SIGNAL_FUNC (on_mnuSave_As_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuSave_As, "activate", accel_group,
                              GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  mnuPrint = gtk_menu_item_new_with_label (_("Print"));
  gtk_widget_set_name (mnuPrint, "mnuPrint");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuPrint", mnuPrint);
  gtk_widget_show (mnuPrint);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuPrint);
  gtk_signal_connect (GTK_OBJECT (mnuPrint), "activate",
                      GTK_SIGNAL_FUNC (on_mnuPrint_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuPrint, "activate", accel_group,
                              GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  

  separator1 = gtk_menu_item_new ();
  gtk_widget_set_name (separator1, "separator1");
  gtk_object_set_data (GTK_OBJECT (frmMain), "separator1", separator1);
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), separator1);

  mnuCheck_Mail = gtk_menu_item_new_with_label (_("Check Mail"));
  gtk_widget_set_name (mnuCheck_Mail, "mnuCheck_Mail");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuCheck_Mail", mnuCheck_Mail);
  gtk_widget_show (mnuCheck_Mail);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuCheck_Mail);
  gtk_signal_connect (GTK_OBJECT (mnuCheck_Mail), "activate",
                      GTK_SIGNAL_FUNC (on_mnuCheck_Mail_activate),
                      NULL);
  /*gtk_widget_add_accelerator (mnuCheck_Mail, "activate", accel_group,
                              GDK_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);*/

  separator2 = gtk_menu_item_new ();
  gtk_widget_set_name (separator2, "separator2");
  gtk_object_set_data (GTK_OBJECT (frmMain), "separator2", separator2);
  gtk_widget_show (separator2);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), separator2);

  mnuExit = gtk_menu_item_new_with_label (_("Exit"));
  gtk_widget_set_name (mnuExit, "mnuExit");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuExit", mnuExit);
  gtk_widget_show (mnuExit);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuExit);
  gtk_signal_connect (GTK_OBJECT (mnuExit), "activate",
                      GTK_SIGNAL_FUNC (on_mnuExit_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuExit, "activate", accel_group,
                              GDK_x, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);


  mnuEdit = gtk_menu_item_new_with_label (_("Edit"));
  gtk_widget_set_name (mnuEdit, "mnuEdit");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuEdit", mnuEdit);
  gtk_widget_show (mnuEdit);
  gtk_container_add (GTK_CONTAINER (menubar), mnuEdit);

  mnuEdit_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuEdit_menu, "mnuEdit_menu");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuEdit_menu", mnuEdit_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuEdit), mnuEdit_menu);

  mnuAddress_Book = gtk_menu_item_new_with_label (_("Address Book"));
  gtk_widget_set_name (mnuAddress_Book, "mnuAddress_Book");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuAddress_Book", mnuAddress_Book);
  gtk_widget_show (mnuAddress_Book);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuAddress_Book);
  gtk_signal_connect (GTK_OBJECT (mnuAddress_Book), "activate",
                      GTK_SIGNAL_FUNC (on_mnuAddress_Book_activate),
                      NULL);

  separator3 = gtk_menu_item_new ();
  gtk_widget_set_name (separator3, "separator3");
  gtk_object_set_data (GTK_OBJECT (frmMain), "separator3", separator3);
  gtk_widget_show (separator3);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), separator3);

  mnuAccounts = gtk_menu_item_new_with_label (_("Accounts"));
  gtk_widget_set_name (mnuAccounts, "mnuAccounts");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuAccounts", mnuAccounts);
  gtk_widget_show (mnuAccounts);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuAccounts);
  gtk_signal_connect (GTK_OBJECT (mnuAccounts), "activate",
                      GTK_SIGNAL_FUNC (on_mnuAccounts_activate),
                      NULL);
  
  separator7 = gtk_menu_item_new ();
  gtk_widget_set_name (separator7, "separator7");
  gtk_object_set_data (GTK_OBJECT (frmMain), "separator7", separator7);
  gtk_widget_show (separator7);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), separator7);
  
  mnuFilters = gtk_menu_item_new_with_label (_("Filters"));
  gtk_widget_set_name (mnuFilters, "mnuFilters");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuFilters", mnuFilters);
  gtk_widget_show (mnuFilters);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuFilters);
  gtk_signal_connect (GTK_OBJECT (mnuFilters), "activate",
                      GTK_SIGNAL_FUNC (on_mnuFilters_activate),
                      NULL);

  separator4 = gtk_menu_item_new ();
  gtk_widget_set_name (separator4, "separator4");
  gtk_object_set_data (GTK_OBJECT (frmMain), "separator4", separator4);
  gtk_widget_show (separator4);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), separator4);

  mnuOptions = gtk_menu_item_new_with_label (_("Options..."));
  gtk_widget_set_name (mnuOptions, "mnuOptions");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuOptions", mnuOptions);
  gtk_widget_show (mnuOptions);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuOptions);
  gtk_signal_connect (GTK_OBJECT (mnuOptions), "activate",
                      GTK_SIGNAL_FUNC (on_mnuOptions_activate),
                      NULL);

  mnuMailboxes = gtk_menu_item_new_with_label (_("Mailboxes"));
  gtk_widget_set_name (mnuMailboxes, "mnuMailboxes");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMailboxes", mnuMailboxes);
  gtk_widget_show (mnuMailboxes);
  gtk_container_add (GTK_CONTAINER (menubar), mnuMailboxes);

  mnuMailboxes_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuMailboxes_menu, "mnuMailboxes_menu");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMailboxes_menu", mnuMailboxes_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuMailboxes), mnuMailboxes_menu);

  mnuMailboxCreate = gtk_menu_item_new_with_label (_("Create..."));
  gtk_widget_set_name (mnuMailboxCreate, "mnuMailboxCreate");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMailboxCreate", mnuMailboxCreate);
  gtk_widget_show (mnuMailboxCreate);
  gtk_container_add (GTK_CONTAINER (mnuMailboxes_menu), mnuMailboxCreate);
  gtk_signal_connect (GTK_OBJECT (mnuMailboxCreate), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMailboxCreate_activate),
                      NULL);

  mnuMailboxDelete = gtk_menu_item_new_with_label (_("Delete"));
  gtk_widget_set_name (mnuMailboxDelete, "mnuMailboxDelete");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMailboxDelete", mnuMailboxDelete);
  gtk_widget_show (mnuMailboxDelete);
  gtk_container_add (GTK_CONTAINER (mnuMailboxes_menu), mnuMailboxDelete);
  gtk_signal_connect (GTK_OBJECT (mnuMailboxDelete), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMailboxDelete_activate),
                      NULL);

  mnuMessage = gtk_menu_item_new_with_label (_("Message"));
  gtk_widget_set_name (mnuMessage, "mnuMessage");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMessage", mnuMessage);
  gtk_widget_show (mnuMessage);
  gtk_container_add (GTK_CONTAINER (menubar), mnuMessage);

  mnuMessage_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuMessage_menu, "mnuMessage_menu");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMessage_menu", mnuMessage_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuMessage), mnuMessage_menu);

  mnuMesgNext = gtk_menu_item_new_with_label (_("Next"));
  gtk_widget_set_name (mnuMesgNext, "mnuMesgNext");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMesgNext", mnuMesgNext);
  gtk_widget_show (mnuMesgNext);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMesgNext);
  gtk_signal_connect (GTK_OBJECT (mnuMesgNext), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMesgNext_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMesgNext, "activate", accel_group,
                              GDK_period, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  mnuMesgPrevious = gtk_menu_item_new_with_label (_("Previous"));
  gtk_widget_set_name (mnuMesgPrevious, "mnuMesgPrevious");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMesgPrevious", mnuMesgPrevious);
  gtk_widget_show (mnuMesgPrevious);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMesgPrevious);
  gtk_signal_connect (GTK_OBJECT (mnuMesgPrevious), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMesgPrevious_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMesgPrevious, "activate", accel_group,
                              GDK_comma, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  separator5 = gtk_menu_item_new ();
  gtk_widget_set_name (separator5, "separator5");
  gtk_object_set_data (GTK_OBJECT (frmMain), "separator5", separator5);
  gtk_widget_show (separator5);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), separator5);

  mnuMesgReply = gtk_menu_item_new_with_label (_("Reply"));
  gtk_widget_set_name (mnuMesgReply, "mnuMesgReply");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMesgReply", mnuMesgReply);
  gtk_widget_show (mnuMesgReply);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMesgReply);
  gtk_signal_connect (GTK_OBJECT (mnuMesgReply), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMesgReply_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMesgReply, "activate", accel_group,
                              GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  mnuMesgForward = gtk_menu_item_new_with_label (_("Forward"));
  gtk_widget_set_name (mnuMesgForward, "mnuMesgForward");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMesgForward", mnuMesgForward);
  gtk_widget_show (mnuMesgForward);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMesgForward);
  gtk_signal_connect (GTK_OBJECT (mnuMesgForward), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMesgForward_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMesgForward, "activate", accel_group,
                              GDK_f, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  separator6 = gtk_menu_item_new ();
  gtk_widget_set_name (separator6, "separator6");
  gtk_object_set_data (GTK_OBJECT (frmMain), "separator6", separator6);
  gtk_widget_show (separator6);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), separator6);

  mnuMesgOpen = gtk_menu_item_new_with_label (_("Open"));
  gtk_widget_set_name (mnuMesgOpen, "mnuMesgOpen");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMesgOpen", mnuMesgOpen);
  gtk_widget_show (mnuMesgOpen);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMesgOpen);
  gtk_signal_connect (GTK_OBJECT (mnuMesgOpen), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMesgOpen_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMesgOpen, "activate", accel_group,
                              GDK_o, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  mnuMesgDelete = gtk_menu_item_new_with_label (_("Delete"));
  gtk_widget_set_name (mnuMesgDelete, "mnuMesgDelete");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMesgDelete", mnuMesgDelete);
  gtk_widget_show (mnuMesgDelete);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMesgDelete);
  gtk_signal_connect (GTK_OBJECT (mnuMesgDelete), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMesgDelete_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMesgDelete, "activate", accel_group,
                              GDK_d, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  mnuMesgExpunge = gtk_menu_item_new_with_label (_("Expunge"));
  gtk_widget_set_name (mnuMesgExpunge, "mnuMesgExpunge");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMesgExpunge", mnuMesgExpunge);
  gtk_widget_show (mnuMesgExpunge);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMesgExpunge);
  gtk_signal_connect (GTK_OBJECT (mnuMesgExpunge), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMesgExpunge_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMesgExpunge, "activate", accel_group,
                              GDK_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  
  mnuCopy_to = gtk_menu_item_new_with_label (_("Copy to..."));
  gtk_widget_set_name (mnuCopy_to, "mnuCopy_to");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuCopy_to", mnuCopy_to);
  gtk_widget_show (mnuCopy_to);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuCopy_to);
  gtk_signal_connect (GTK_OBJECT (mnuCopy_to), "activate",
                      GTK_SIGNAL_FUNC (on_mnuCopy_to_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuCopy_to, "activate", accel_group,
                              GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  
  mnuMove_to = gtk_menu_item_new_with_label (_("Move to..."));
  gtk_widget_set_name (mnuMove_to, "mnuMove_to");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuMove_to", mnuMove_to);
  gtk_widget_show (mnuMove_to);
  gtk_container_add (GTK_CONTAINER (mnuMessage_menu), mnuMove_to);
  gtk_signal_connect (GTK_OBJECT (mnuMove_to), "activate",
                      GTK_SIGNAL_FUNC (on_mnuMove_to_activate),
                      NULL);
  gtk_widget_add_accelerator (mnuMove_to, "activate", accel_group,
                              GDK_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  mnuMime = gtk_menu_item_new_with_label (_("Attachments"));
  gtk_widget_set_name(mnuMime,"mnuMime");
  gtk_object_set_data(GTK_OBJECT(frmMain),"mnuMime",mnuMime);
  gtk_widget_show(mnuMime);
  gtk_container_add(GTK_CONTAINER(menubar),mnuMime);

  mnuMime_menu = gtk_menu_new();
  gtk_widget_set_name(mnuMime_menu,"mnuMime_menu");
  gtk_object_set_data(GTK_OBJECT(frmMain),"mnuMime_menu",mnuMime_menu);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(mnuMime),mnuMime_menu);

  mnuMimeSave = gtk_menu_item_new_with_label (_("Save"));
  gtk_widget_set_name(mnuMimeSave,"mnuMimeSave");
  gtk_object_set_data(GTK_OBJECT(frmMain),"mnuMimeSave",mnuMimeSave);
  gtk_widget_show(mnuMimeSave);
  gtk_container_add(GTK_CONTAINER(mnuMime_menu),mnuMimeSave);
  gtk_signal_connect(GTK_OBJECT(mnuMimeSave),"activate",GTK_SIGNAL_FUNC(on_mnuMimeSave_activate),NULL);

  mnuHelp = gtk_menu_item_new_with_label (_("Help"));
  gtk_widget_set_name (mnuHelp, "mnuHelp");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuHelp", mnuHelp);
  gtk_widget_show (mnuHelp);
  gtk_container_add (GTK_CONTAINER (menubar), mnuHelp);
  gtk_menu_item_right_justify (GTK_MENU_ITEM (mnuHelp));

  mnuHelp_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuHelp_menu, "mnuHelp_menu");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuHelp_menu", mnuHelp_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuHelp), mnuHelp_menu);

  mnuAbout = gtk_menu_item_new_with_label (_("About..."));
  gtk_widget_set_name (mnuAbout, "mnuAbout");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuAbout", mnuAbout);
  gtk_widget_show (mnuAbout);
  gtk_container_add (GTK_CONTAINER (mnuHelp_menu), mnuAbout);
  gtk_signal_connect (GTK_OBJECT (mnuAbout), "activate",
                      GTK_SIGNAL_FUNC (on_mnuAbout_activate),
                      NULL);

  mnuHowto = gtk_menu_item_new_with_label (_("Howto"));
  gtk_widget_set_name (mnuHowto, "mnuHowto");
  gtk_object_set_data (GTK_OBJECT (frmMain), "mnuHowto", mnuHowto);
  gtk_widget_show (mnuHowto);
  gtk_container_add (GTK_CONTAINER (mnuHelp_menu), mnuHowto);
  gtk_signal_connect (GTK_OBJECT (mnuHowto), "activate",
                      GTK_SIGNAL_FUNC (on_mnuHowto_activate),
                      NULL);

  hboxBody = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hboxBody, "hboxBody");
  gtk_object_set_data (GTK_OBJECT (frmMain), "hboxBody", hboxBody);
  gtk_widget_show (hboxBody);
  gtk_table_attach (GTK_TABLE (table), hboxBody, 0, 18, 11, 22,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

  txtMessage = gtk_text_new (NULL, NULL);
  gtk_text_set_word_wrap (GTK_TEXT(txtMessage), TRUE);
  gtk_widget_set_name (txtMessage, "txtMessage");
  gtk_object_set_data (GTK_OBJECT (frmMain), "txtMessage", txtMessage);
  gtk_widget_show (txtMessage);
  gtk_box_pack_start (GTK_BOX (hboxBody), txtMessage, TRUE, TRUE, 0);

  vsbMessage = gtk_vscrollbar_new (GTK_TEXT(txtMessage)->vadj);
  gtk_widget_set_name (vsbMessage, "vsbMessage");
  gtk_object_set_data (GTK_OBJECT (frmMain), "vsbMessage", vsbMessage);
  gtk_widget_show (vsbMessage);
  gtk_box_pack_start (GTK_BOX (hboxBody), vsbMessage, FALSE, TRUE, 0);

  scrlBoxes = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrlBoxes, "scrlBoxes");
  gtk_object_set_data (GTK_OBJECT (frmMain), "scrlBoxes", scrlBoxes);
  gtk_widget_show (scrlBoxes);
  gtk_table_attach (GTK_TABLE (table), scrlBoxes, 0, 4, 3, 11,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrlBoxes), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clistBoxes = gtk_clist_new (1);
  gtk_widget_set_name (clistBoxes, "clistBoxes");
  gtk_object_set_data (GTK_OBJECT (frmMain), "clistBoxes", clistBoxes);
  gtk_widget_show (clistBoxes);
  gtk_container_add (GTK_CONTAINER (scrlBoxes), clistBoxes);
  gtk_signal_connect_object (GTK_OBJECT (clistBoxes), "select_row",
                             GTK_SIGNAL_FUNC (on_clistBoxes_select_row),
                             GTK_OBJECT (clistBoxes));
  gtk_signal_connect_object (GTK_OBJECT (clistBoxes), "button_press_event",
                             GTK_SIGNAL_FUNC (on_clistBoxes_clicked),
                             GTK_OBJECT (clistBoxes));
  gtk_signal_connect(GTK_OBJECT(clistBoxes),"drag_data_received",GTK_SIGNAL_FUNC(on_clistBoxes_drag_data_recieved),NULL);
  gtk_drag_dest_set(clistBoxes,GTK_DEST_DEFAULT_ALL,target,2,GDK_ACTION_MOVE | GDK_ACTION_COPY);
  gtk_clist_set_column_width (GTK_CLIST (clistBoxes), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (clistBoxes));

  label11 = gtk_label_new ("label11");
  gtk_widget_set_name (label11, "label11");
  gtk_object_set_data (GTK_OBJECT (frmMain), "label11", label11);
  gtk_widget_show (label11);
  gtk_clist_set_column_widget (GTK_CLIST (clistBoxes), 0, label11);

  scrlMail = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrlMail, "scrlMail");
  gtk_object_set_data (GTK_OBJECT (frmMain), "scrlMail", scrlMail);
  gtk_widget_show (scrlMail);
  gtk_table_attach (GTK_TABLE (table), scrlMail, 4, 18, 3, 11,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrlMail), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

  clistMail = gtk_clist_new (4);
  gtk_clist_set_selection_mode(GTK_CLIST(clistMail), GTK_SELECTION_EXTENDED);
  gtk_widget_set_name (clistMail, "clistMail");
  gtk_object_set_data (GTK_OBJECT (frmMain), "clistMail", clistMail);
  gtk_widget_show (clistMail);
  gtk_container_add (GTK_CONTAINER (scrlMail), clistMail);
  gtk_signal_connect_object (GTK_OBJECT (clistMail), "select_row",
                             GTK_SIGNAL_FUNC (on_clistMail_select_row),
                             GTK_OBJECT (clistMail));
  gtk_signal_connect_object (GTK_OBJECT (clistMail), "button_press_event",
                             GTK_SIGNAL_FUNC (on_clistMail_clicked),
                             GTK_OBJECT (clistMail));
  gtk_signal_connect_object (GTK_OBJECT (clistMail), "click_column",
                             GTK_SIGNAL_FUNC (on_clistMail_click_column),
                             GTK_OBJECT (clistMail));
  gtk_signal_connect(GTK_OBJECT(clistMail),"drag_data_get",GTK_SIGNAL_FUNC(on_clistMail_drag_data_get),NULL);
  gtk_drag_source_set(clistMail,GDK_BUTTON1_MASK,target,2,GDK_ACTION_MOVE | GDK_ACTION_COPY);
  gtk_clist_set_column_width (GTK_CLIST (clistMail), 0, 14);
  gtk_clist_set_column_width (GTK_CLIST (clistMail), 1, 120);
  gtk_clist_set_column_width (GTK_CLIST (clistMail), 2, 225);
  gtk_clist_set_column_width (GTK_CLIST (clistMail), 3, 75);
  gtk_clist_column_titles_show (GTK_CLIST (clistMail));
  /* set the default compare function */
  gtk_clist_set_compare_func (GTK_CLIST (clistMail), id_compare);
  
  scrlMime = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrlMime, "scrlMime");
  gtk_object_set_data (GTK_OBJECT (frmMain), "scrlMime", scrlMime);
  gtk_widget_show (scrlMime);
  gtk_table_attach (GTK_TABLE (table), scrlMime, 0, 18, 22, 23,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrlMime), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clistMime=gtk_clist_new(4);
  gtk_widget_set_name(clistMime,"clistMime");
  gtk_object_set_data(GTK_OBJECT(frmMain),"clistMime",clistMime);
  gtk_container_add(GTK_CONTAINER(scrlMime),clistMime);
  gtk_clist_set_column_width(GTK_CLIST(clistMime),0,20);
  gtk_clist_set_column_width(GTK_CLIST(clistMime),1,190);
  gtk_clist_set_column_width(GTK_CLIST(clistMime),2,225);
  gtk_clist_set_column_width(GTK_CLIST(clistMime),3,50);
  gtk_signal_connect(GTK_OBJECT(clistMime),"select_row",GTK_SIGNAL_FUNC(on_clistMime_select_row),NULL);
  gtk_widget_show(clistMime);
  
  lblMesgID = gtk_label_new (_("id"));
  gtk_widget_set_name (lblMesgID, "lblMesgID");
  gtk_object_set_data (GTK_OBJECT (frmMain), "lblMesgID", lblMesgID);
  gtk_widget_show (lblMesgID);
  gtk_clist_set_column_widget (GTK_CLIST (clistMail), 0, lblMesgID);
  
  lblSender = gtk_label_new (_("Sender"));
  gtk_widget_set_name (lblSender, "lblSender");
  gtk_object_set_data (GTK_OBJECT (frmMain), "lblSender", lblSender);
  gtk_widget_show (lblSender);
  gtk_clist_set_column_widget (GTK_CLIST (clistMail), 1, lblSender);

  lblSubject = gtk_label_new (_("Subject"));
  gtk_widget_set_name (lblSubject, "lblSubject");
  gtk_object_set_data (GTK_OBJECT (frmMain), "lblSubject", lblSubject);
  gtk_widget_show (lblSubject);
  gtk_clist_set_column_widget (GTK_CLIST (clistMail), 2, lblSubject);

  lblDate = gtk_label_new (_("Date"));
  gtk_widget_set_name (lblDate, "lblDate");
  gtk_object_set_data (GTK_OBJECT (frmMain), "lblDate", lblDate);
  gtk_widget_show (lblDate);
  gtk_clist_set_column_widget (GTK_CLIST (clistMail), 3, lblDate);
  
  handlebox = gtk_handle_box_new();
  gtk_table_attach(GTK_TABLE(table), handlebox, 0, 18, 1, 3,
         (GtkAttachOptions) GTK_EXPAND | GTK_FILL, FALSE, 0, 0);
  gtk_widget_show(handlebox);
  
  switch (texticons)
  {
     case (TB_TEXT):
        toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
        break;
     case (TB_ICONS):
        toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
        break;
     case (TB_BOTH):
        toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
        break;
  }

  gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);

  gtk_container_set_border_width (GTK_CONTAINER (toolbar), 2);
  gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar), 1);  
  gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
  gtk_widget_show(toolbar);
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, check_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdCheckMail = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Check"), _("Check for new mail"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdCheckMail_clicked), toolbar);
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, new_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdNew = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Compose"), _("Compose Message"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdNew_clicked), toolbar);
  /*gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));*/
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, msave_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdSave = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Save"), _("Save Message"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdSave_clicked), toolbar);
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, print_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdPrint = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Print"), _("Print Message"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdPrintToolbar_clicked), toolbar);
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, reply_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdReply = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Reply"), _("Reply"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdReply_clicked), toolbar);
  /*gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));*/
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, fwd_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdFwd = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Forward"), _("Forward"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdFwd_clicked), toolbar);
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, maddr_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdAddrbk = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Address"), _("Address book"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdAddrbk_clicked), toolbar);
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, del_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdDel = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Delete"), _("Delete Message"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdDel_clicked), toolbar);
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, larrow_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdPrevious = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Previous"), _("Previous Message"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdPrevious_clicked), toolbar);
  /*gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));*/
  
  icon = gdk_pixmap_create_from_xpm_d (frmMain->window, &mask,
               &frmMain->style->white, rarrow_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdNext = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
               GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Next"), _("Next Message"), "Private",
               iconw, GTK_SIGNAL_FUNC(on_cmdNext_clicked), toolbar);
  /*gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));*/
  
  return frmMain;
}

GtkWidget* create_frmMainSaveAs ()
{
  GtkWidget *frmMainSaveAs;
  GtkWidget *cmdMainSaveAs_OK;
  GtkWidget *cmdMainSaveAs_Cancel;

  frmMainSaveAs = gtk_file_selection_new (_("Save Email As..."));
  gtk_widget_set_name (frmMainSaveAs, "frmMainSaveAs");
  gtk_object_set_data (GTK_OBJECT (frmMainSaveAs), "frmMainSaveAs", 
		       frmMainSaveAs);
  gtk_container_border_width (GTK_CONTAINER (frmMainSaveAs), 10);
  gtk_signal_connect_object (GTK_OBJECT (frmMainSaveAs), "destroy",
                             GTK_SIGNAL_FUNC (on_frmMainSaveAs_destroy),
                             GTK_OBJECT (frmMainSaveAs));

  cmdMainSaveAs_OK = GTK_FILE_SELECTION (frmMainSaveAs)->ok_button;
  gtk_widget_set_name (cmdMainSaveAs_OK, "cmdMainSaveAs_OK");
  gtk_object_set_data (GTK_OBJECT (frmMainSaveAs), "cmdMainSaveAs_OK", 
		       cmdMainSaveAs_OK);
  gtk_widget_show (cmdMainSaveAs_OK);
  GTK_WIDGET_SET_FLAGS (cmdMainSaveAs_OK, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (cmdMainSaveAs_OK), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdMainSaveAs_OK_clicked),
                             GTK_OBJECT (cmdMainSaveAs_OK));

  cmdMainSaveAs_Cancel = GTK_FILE_SELECTION (frmMainSaveAs)->cancel_button;
  gtk_widget_set_name (cmdMainSaveAs_Cancel, "cmdMainSaveAs_Cancel");
  gtk_object_set_data (GTK_OBJECT (frmMainSaveAs), "cmdMainSaveAs_Cancel", 
		       cmdMainSaveAs_Cancel);
  gtk_widget_show (cmdMainSaveAs_Cancel);
  GTK_WIDGET_SET_FLAGS (cmdMainSaveAs_Cancel, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (cmdMainSaveAs_Cancel), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdMainSaveAs_Cancel_clicked),
                             GTK_OBJECT (cmdMainSaveAs_Cancel));

  return frmMainSaveAs;
}

GtkWidget* create_dlgNewMailbox ()
{
  GtkWidget *dlgNewMailbox;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *lblBah;
  GtkWidget *txtNewMailbox;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbox1;
  GtkWidget *cmdDlgNewMailboxOK;
  GtkWidget *cmdDlgNewMailboxCancel;

  dlgNewMailbox = gtk_dialog_new ();
  gtk_widget_set_name (dlgNewMailbox, "dlgNewMailbox");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "dlgNewMailbox", 
		       dlgNewMailbox);
  gtk_widget_set_usize (dlgNewMailbox, 200, 125);
  gtk_signal_connect_object (GTK_OBJECT (dlgNewMailbox), "destroy",
                             GTK_SIGNAL_FUNC (on_dlgNewMailbox_destroy),
                             GTK_OBJECT (dlgNewMailbox));
  gtk_window_set_title (GTK_WINDOW (dlgNewMailbox), _("New Mailbox"));
  gtk_window_set_policy (GTK_WINDOW (dlgNewMailbox), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (dlgNewMailbox)->vbox;
  gtk_widget_set_name (dialog_vbox1, "dialog_vbox1");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "dialog_vbox1", 
		       dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox1, "vbox1");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  lblBah = gtk_label_new (_("Enter the name of\nthe Mailbox you wish\nto create."));
  gtk_widget_set_name (lblBah, "lblBah");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "lblBah", lblBah);
  gtk_widget_show (lblBah);
  gtk_box_pack_start (GTK_BOX (vbox1), lblBah, FALSE, FALSE, 0);

  txtNewMailbox = gtk_entry_new ();
  gtk_widget_set_name (txtNewMailbox, "txtNewMailbox");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "txtNewMailbox", 
		       txtNewMailbox);
  gtk_widget_show (txtNewMailbox);
  gtk_box_pack_start (GTK_BOX (vbox1), txtNewMailbox, TRUE, TRUE, 0);

  dialog_action_area1 = GTK_DIALOG (dlgNewMailbox)->action_area;
  gtk_widget_set_name (dialog_action_area1, "dialog_action_area1");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "dialog_action_area1", 
		       dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbox1 = gtk_hbox_new (TRUE, 0);
  gtk_widget_set_name (hbox1, "hbox1");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox1, TRUE, TRUE, 0);

  cmdDlgNewMailboxOK = gtk_button_new_with_label (_("OK"));
  gtk_widget_set_name (cmdDlgNewMailboxOK, "cmdDlgNewMailboxOK");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "cmdDlgNewMailboxOK", 
		       cmdDlgNewMailboxOK);
  gtk_widget_show (cmdDlgNewMailboxOK);
  gtk_box_pack_start (GTK_BOX (hbox1), cmdDlgNewMailboxOK, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cmdDlgNewMailboxOK), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdDlgNewMailboxOK_clicked),
                             GTK_OBJECT (cmdDlgNewMailboxOK));

  cmdDlgNewMailboxCancel = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_set_name (cmdDlgNewMailboxCancel, "cmdDlgNewMailboxCancel");
  gtk_object_set_data (GTK_OBJECT (dlgNewMailbox), "cmdDlgNewMailboxCancel", 
		       cmdDlgNewMailboxCancel);
  gtk_widget_show (cmdDlgNewMailboxCancel);
  gtk_box_pack_start (GTK_BOX (hbox1), cmdDlgNewMailboxCancel, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cmdDlgNewMailboxCancel), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdDlgNewMailboxCancel_clicked),
                             GTK_OBJECT (cmdDlgNewMailboxCancel));

  return dlgNewMailbox;
}

GtkWidget* create_dlgCopyMove (gchar *title)
{
  GtkWidget *dlgCopyMove;
  GtkWidget *fixed;
  GtkWidget *cmboMailbox;
  GList *cmboMailbox_items = NULL;
  GtkWidget *lblMailbox;
  GtkWidget *cmdCopyMoveOK;
  GtkWidget *cmdCopyMoveCancel;
  GtkWidget *clistMailboxes = get_widget(frmMain, "clistBoxes");
  gint i;
  gchar *mailbox;

  dlgCopyMove = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (dlgCopyMove, "dlgCopyMove");
  gtk_object_set_data (GTK_OBJECT (dlgCopyMove), "dlgCopyMove", dlgCopyMove);
  gtk_widget_set_usize (dlgCopyMove, 230, 115);
  gtk_signal_connect_object (GTK_OBJECT (dlgCopyMove), "destroy",
                             GTK_SIGNAL_FUNC (on_dlgCopyMove_destroy),
                             GTK_OBJECT (dlgCopyMove));
  gtk_window_set_title (GTK_WINDOW (dlgCopyMove), _(title));
  gtk_window_set_policy (GTK_WINDOW (dlgCopyMove), FALSE, FALSE, FALSE);

  fixed = gtk_fixed_new ();
  gtk_widget_set_name (fixed, "fixed");
  gtk_object_set_data (GTK_OBJECT (dlgCopyMove), "fixed", fixed);
  gtk_widget_show (fixed);
  gtk_container_add (GTK_CONTAINER (dlgCopyMove), fixed);

  cmboMailbox = gtk_combo_new ();
  gtk_widget_set_name (cmboMailbox, "cmboMailbox");
  gtk_object_set_data (GTK_OBJECT (dlgCopyMove), "cmboMailbox", cmboMailbox);
  gtk_widget_show (cmboMailbox);
  gtk_fixed_put (GTK_FIXED (fixed), cmboMailbox, 24, 32);
  gtk_widget_set_uposition (cmboMailbox, 24, 32);
  gtk_widget_set_usize (GTK_COMBO (cmboMailbox)->entry, 168, 24);
  gtk_widget_set_usize (cmboMailbox, 184, 24);
  for (i = 0; i < NumMailboxes; i++)
  {
     gtk_clist_get_text(GTK_CLIST(clistMailboxes), i, 0, &mailbox);
     cmboMailbox_items = g_list_append (cmboMailbox_items, mailbox);
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (cmboMailbox), cmboMailbox_items);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(cmboMailbox)->entry), FALSE);
  g_list_free (cmboMailbox_items);

  lblMailbox = gtk_label_new (_("Choose mailbox"));
  gtk_widget_set_name (lblMailbox, "lblMailbox");
  gtk_object_set_data (GTK_OBJECT (dlgCopyMove), "lblMailbox", lblMailbox);
  gtk_widget_show (lblMailbox);
  gtk_fixed_put (GTK_FIXED (fixed), lblMailbox, 8, 8);
  gtk_widget_set_uposition (lblMailbox, 8, 8);
  gtk_widget_set_usize (lblMailbox, 216, 16);

  cmdCopyMoveOK = gtk_button_new_with_label (_("OK"));
  gtk_widget_set_name (cmdCopyMoveOK, "cmdCopyMoveOK");
  gtk_object_set_data (GTK_OBJECT (dlgCopyMove), "cmdCopyMoveOK", cmdCopyMoveOK);
  gtk_widget_show (cmdCopyMoveOK);
  gtk_fixed_put (GTK_FIXED (fixed), cmdCopyMoveOK, 24, 72);
  gtk_widget_set_uposition (cmdCopyMoveOK, 24, 72);
  gtk_widget_set_usize (cmdCopyMoveOK, 72, 32);
  gtk_signal_connect_object (GTK_OBJECT (cmdCopyMoveOK), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdCopyMoveOK_clicked),
                             GTK_OBJECT (cmdCopyMoveOK));

  cmdCopyMoveCancel = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_set_name (cmdCopyMoveCancel, "cmdCopyMoveCancel");
  gtk_object_set_data (GTK_OBJECT (dlgCopyMove), "cmdCopyMoveCancel", cmdCopyMoveCancel);
  gtk_widget_show (cmdCopyMoveCancel);
  gtk_fixed_put (GTK_FIXED (fixed), cmdCopyMoveCancel, 136, 72);
  gtk_widget_set_uposition (cmdCopyMoveCancel, 136, 72);
  gtk_widget_set_usize (cmdCopyMoveCancel, 72, 32);
  gtk_signal_connect_object (GTK_OBJECT (cmdCopyMoveCancel), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdCopyMoveCancel_clicked),
                             GTK_OBJECT (cmdCopyMoveCancel));

  return dlgCopyMove;
}

void on_frmMain_destroy (GtkObject *object, gpointer user_data)
{
   if (empty_on_exit)
   {
      /* emtpy the contents of "Deleted items" */
      mailbox_delete("Deleted items");
      mailbox_create("Deleted items");
   }
   
   rc_save_defaults(); /* so the window size, and column sizes is saved */
   global_free();    /* free our globals variables */
   gtk_main_quit();  /* exit */
}

void on_mnuNew_Message_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdNew_clicked(NULL, NULL);
}

void on_cmdNew_clicked (GtkWidget *widget, gpointer data)
{
   if (!ComposerOpen)
   {
      frmComposer = create_frmComposer(SenderAddr, NULL, NULL, NULL);
      gtk_widget_show(frmComposer);
      ComposerOpen = 1;
   }
}

void on_mnuSave_As_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdSave_clicked(NULL, NULL);
}

void on_cmdSave_clicked (GtkWidget *widget, gpointer data)
{
   if (!SaveAsOpen && cMesg >= 0)
   {
      frmMainSaveAs = create_frmMainSaveAs();
      gtk_widget_show(frmMainSaveAs);
      SaveAsOpen = 1;
   }
}

void on_mnuPrint_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdPrintToolbar_clicked(NULL, NULL);
}

void on_cmdPrintToolbar_clicked (GtkWidget *widget, gpointer data)
{
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail  = get_widget(frmMain, "clistMail");
   gchar *mailbox = NULL;
   gchar *mesgid  = NULL;
   gchar *mesg    = NULL;
   gchar from[256], date[256], subject[256];
   gchar header[1024];

   if (!PrintOpen && cMesg >= 0)
   {
      gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);
      gtk_clist_get_text(GTK_CLIST(clistMail), cMesg, 0, &mesgid);
      mesg = mailbox_get_mesg(atoi(mesgid), mailbox);
      strncpy(from, mailbox_get_mesg_from(mesg), 256);
		strncpy(date, mailbox_get_mesg_date(mesg), 256);
		strncpy(subject, mailbox_get_mesg_subject(mesg), 256);
      g_snprintf(header, 1024, "From: %s\r\nDate: %s\r\nSubject: %s\r\n", 
                 from, date, subject);
		trim_header(mesg, hdr_trim_level);

      frmPrint = create_frmPrint(header, mesg);
      g_free(mesg);
      gtk_widget_show(frmPrint);
      PrintOpen = 1;
   }
}

void on_cmdCheckMail_clicked (GtkWidget *widget, gpointer data)
{
   mail_check (HIDE_STATUS_DIALOG);
}

void on_mnuCheck_Mail_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdCheckMail_clicked(NULL, NULL);
}

gint periodic_mail_check (gpointer data)
{
   mail_check (HIDE_STATUS_DIALOG);
   return 1;
}

void mail_check (gboolean show_dialog)
{
   gint i;
   gint err;
   gchar reason[50];
   gchar buffer[256];

   if (!fetching_mail)
   {
      fetching_mail = TRUE;

      for (i = 0; i < NumAccounts; i++)
      {
         g_snprintf(buffer, 255, "%s", AccountsList[i].server.hostname);
	      if (show_dialog)
         {
	         frmStatus = create_frmStatus(buffer);
	         gtk_widget_show(frmStatus);
	      }
         fprintf(stdout, "Retrieving mail from: %s:%d...",
                 AccountsList[i].server.hostname,
                 AccountsList[i].server.port);
         fflush(stdout);
         err = fetchmail(&AccountsList[i].server, AccountsList[i].login, 
                         AccountsList[i].passwd, 
                         get_mailbox(AccountsList[i].mailbox),
                         AccountsList[i].delemail, show_dialog);
	      if (show_dialog)
	         gtk_widget_destroy(frmStatus);

         if(!err)
         {
            pop3error(reason, 49);
            fprintf(stderr, "failed.\nSpruce%s\n", reason);
            fflush(stdout);
         }
         else
         {
            fprintf(stdout, "done.\n");
            fflush(stdout);
         }
      }

      fetching_mail = FALSE;
      refresh_mail_list(STAY);
   }
}

void on_mnuExit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_frmMain_destroy(NULL,NULL);
}

void on_mnuAddress_Book_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdAddrbk_clicked(NULL, NULL);
}

void on_cmdAddrbk_clicked (GtkWidget *widget, gpointer data)
{
   if (!AddrBookOpen)
   {
      frmAddrBook = create_frmAddrBook(NULL);
      gtk_widget_show(frmAddrBook);
      AddrBookOpen = 1;
   }
}

void on_mnuAccounts_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   if (!AccountsOpen)
   {
      frmAccounts = create_frmAccounts(TRUE);
      gtk_widget_show(frmAccounts);
      AccountsOpen = 1;
   }
}

void on_mnuFilters_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   if(!FiltersOpen)
   {
      frmFilters = create_frmFilters();
      gtk_widget_show(frmFilters);
      filters_refresh();
      FiltersOpen = 1;
   }
}

void on_mnuOptions_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   if (!OptionsOpen)
   {
      frmOptions = create_frmOptions();
      gtk_widget_show(frmOptions);
      OptionsOpen = 1;
   }
}

void on_mnuMailboxCreate_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   if (!MbxNewOpen && NumMailboxes)
   {
      dlgNewMailbox = create_dlgNewMailbox();
      gtk_widget_show(dlgNewMailbox);
      MbxNewOpen = 1;
   }
}

void on_mnuMailboxDelete_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   GtkWidget *clist = get_widget(frmMain, "clistBoxes");
   gchar *mailbox;

   if (cMbox >= 0)
   {
      gtk_clist_get_text(GTK_CLIST(clist), cMbox, 0, &mailbox);

      mailbox_delete(mailbox);

      gtk_clist_freeze(GTK_CLIST(clist));
      gtk_clist_remove(GTK_CLIST(clist), cMbox);

      NumMailboxes--;
      if (cMbox >= NumMailboxes)
         cMbox = NumMailboxes - 1;

      gtk_clist_select_row(GTK_CLIST(clist), cMbox, 0);

      rc_save_defaults();

      gtk_clist_thaw(GTK_CLIST(clist));

      /*refresh_mailboxes(STAY);*/
   }
}

void on_mnuMesgNext_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdNext_clicked(NULL, NULL);
}

void on_cmdNext_clicked (GtkWidget *widget, gpointer data)
{
   GtkWidget *clist = get_widget(frmMain, "clistMail");
   GtkVisibility visible;

   if (cMesg < NumMesgs - 1)
   {
      gtk_clist_unselect_row(GTK_CLIST(clist), cMesg, 0);
      gtk_clist_select_row(GTK_CLIST(clist), cMesg + 1, 0);
      visible = gtk_clist_row_is_visible(GTK_CLIST(clist), cMesg);
      if (visible != GTK_VISIBILITY_FULL)
         gtk_clist_moveto(GTK_CLIST(clist), cMesg, 0, 1.0, 0.0);
   }
}

void on_mnuMesgPrevious_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdPrevious_clicked(NULL, NULL);
}

void on_cmdPrevious_clicked (GtkWidget *widget, gpointer data)
{
   GtkWidget *clist = get_widget(frmMain, "clistMail");
   GtkVisibility visible;

   if (cMesg > 0)
   {
      gtk_clist_unselect_row(GTK_CLIST(clist), cMesg, 0);
      gtk_clist_select_row(GTK_CLIST(clist), cMesg - 1, 0);
      visible = gtk_clist_row_is_visible(GTK_CLIST(clist), cMesg);
      if (visible != GTK_VISIBILITY_FULL)
         gtk_clist_moveto(GTK_CLIST(clist), cMesg, 0, 0.0, 0.0);
   }
}

void on_mnuMesgReply_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdReply_clicked (NULL, NULL);
}

void on_cmdReply_clicked (GtkWidget *widget, gpointer data)
{
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   GtkWidget *clist   = get_widget(frmMain, "clistMail");
   GtkWidget *txtBody = get_widget(frmMain, "txtMessage");
   gchar *recipient   = NULL;
   gchar *OrigSubject = NULL;
   gchar *subject     = NULL;
   gchar *date        = NULL;
   gchar *body        = NULL;
   gchar *mod_body    = NULL;
   gchar *mesg        = NULL;
   gchar *replyto     = NULL;
   gchar *mailbox     = NULL;
   gchar *mesgid      = NULL;

   if (!ComposerOpen && cMesg >= 0)
   {
      gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);
      gtk_clist_get_text(GTK_CLIST(clist), cMesg, 0, &mesgid);
      gtk_clist_get_text(GTK_CLIST(clist), cMesg, 1, &recipient);
      gtk_clist_get_text(GTK_CLIST(clist), cMesg, 2, &OrigSubject);
      gtk_clist_get_text(GTK_CLIST(clist), cMesg, 3, &date);

      mesg = mailbox_get_mesg(atoi(mesgid), mailbox);
      replyto = mailbox_get_mesg_replyto(mesg);
      
      body = gtk_editable_get_chars(GTK_EDITABLE(txtBody), 0, -1);

      mod_body = quote_body(body, recipient, date);

      if (OrigSubject == NULL)
      {
         /* OrigSubject is NULL */
         subject = g_malloc(strlen("Re: ") + 1);
         sprintf(subject, "Re: ");
      }
      else
      {
         subject = g_malloc(strlen("Re: ") + strlen(OrigSubject) + 1);
         sprintf(subject, "Re: %s", OrigSubject);
      }

      if (replyto == NULL || strstrcase(recipient, replyto) != NULL)
      {
         frmComposer = create_frmComposer(SenderAddr, recipient, subject, mod_body);
      }
      else
      {
         frmComposer = create_frmComposer(SenderAddr, replyto, subject, mod_body);
      }
      gtk_widget_show(frmComposer);

      g_free(subject);
      g_free(mod_body);
      g_free(body);
      g_free(mesg);

      ComposerOpen = 1;
   }
}

void on_mnuMesgForward_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdFwd_clicked(NULL, NULL);
}

void on_cmdFwd_clicked (GtkWidget *widget, gpointer data)
{
   GtkWidget *clist   = get_widget(frmMain, "clistMail");
   GtkWidget *txtBody = get_widget(frmMain, "txtMessage");
   gchar *OrigSubject = NULL;
   gchar *subject     = NULL;
   gchar *body        = NULL;
   gchar *mod_body    = NULL;
   gchar *sender      = NULL;
   gchar *date        = NULL;

   if (!ComposerOpen && cMesg >= 0)
   {
      gtk_clist_get_text(GTK_CLIST(clist), cMesg, 1, &sender);
      gtk_clist_get_text(GTK_CLIST(clist), cMesg, 2, &OrigSubject);
      gtk_clist_get_text(GTK_CLIST(clist), cMesg, 3, &date);

      if (OrigSubject == NULL)
      {
         /* OrigSubject is NULL */
         subject = g_malloc(strlen("Fw: ") + 1);
         sprintf(subject, "Fw: ");
      }
      else
      {
         subject = g_malloc(strlen("Fw: ") + strlen(OrigSubject) + 1);
         sprintf(subject, "Fw: %s", OrigSubject);
      }

      body = gtk_editable_get_chars(GTK_EDITABLE(txtBody), 0, -1);

      mod_body = quote_body(body, sender, date);

      frmComposer = create_frmComposer(SenderAddr, NULL, subject, mod_body);
      gtk_widget_show(frmComposer);

      g_free(subject);
      g_free(mod_body);
      g_free(body);

      ComposerOpen = 1;
   }
}

void on_mnuMesgOpen_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail  = get_widget(frmMain, "clistMail");
   gchar *mesg = NULL;
   gchar *mailbox;
   gchar *mesgid;

   gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);

   if (!MesgReaderOpen && cMesg >= 0)
   {
      gtk_clist_get_text(GTK_CLIST(clistMail), cMesg, 0, &mesgid);
      mesg = mailbox_get_mesg(atoi(mesgid), mailbox);

      frmMesgReader = create_frmMesgReader(mesg, mailbox_get_mesg_subject(mesg));
      gtk_widget_show(frmMesgReader);
      MesgReaderOpen = 1;
      g_free(mesg);
   }
}

void on_mnuMesgDelete_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   on_cmdDel_clicked(NULL, NULL);
}

void on_cmdDel_clicked (GtkWidget *widget, gpointer data)
{
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail  = get_widget(frmMain, "clistMail");
   gchar *mailbox;
   gchar *mesgid;
   GList *selection;
   guint num = 0, *mesg_nums, i = 0;
   gint y, temp;
   

   gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);

   selection = GTK_CLIST(clistMail)->selection;
   num = g_list_length(selection);
   if (num <= 0)	/* no messages, so skip out */
      return;
      
   mesg_nums = g_malloc0(sizeof(gint) * num);

   selection = GTK_CLIST(clistMail)->selection;
   while(selection != NULL)
   {
      gtk_clist_get_text(GTK_CLIST(clistMail), (gint)selection->data, 0, &mesgid);
      mesg_nums[i] = atoi(mesgid);
      selection = selection->next;
      i++;
   }

   /* gotta sort the ids */
   y = 0;
   while(y == 0)
   {
      y = 1;
      i = 0;
      while(i < num - 1)
      {
         if(mesg_nums[i] > mesg_nums[i+1])
         {
            temp = mesg_nums[i];
            mesg_nums[i] = mesg_nums[i+1];
            mesg_nums[i+1] = temp;
            y = 0;
         }
         i++;
      }
   }

   if (num > 0)
   {
      if (strcmp(mailbox, "Deleted items"))
      {
         /* the user is not in the "Deleted items" mailbox */
         /* so we want to move it/them to the "Deleted items" box */
         mailbox_move_mesgs(num, mesg_nums, mailbox, "Deleted items");
      }
      else
      {
         /* the user is in the "Deleted items" mailbox */
         /* so we should expunge the message(s) */
         mailbox_del_mesgs(num, mesg_nums, mailbox);
      }
      clistMail_update();
      mailbox_refresh_mailpos(num, mesg_nums, NumMesgs, mailbox);
      cMesg--;
      i = 0;
      while(i < GTK_CLIST(clistMail)->rows)
      {
         gtk_clist_unselect_row(GTK_CLIST(clistMail), i, 0);
         i++;
      }
      if(NumMesgs > 0)
      {
         gtk_clist_select_row(GTK_CLIST(clistMail), cMesg+1, 0);
      }
      /*refresh_mail_list(STAY);*/
   }

   if (NumMesgs == 0)
   {
      cMesg = -1;
      refresh_txtMessage();
   }

   g_free(mesg_nums);
}

void on_mnuMesgExpunge_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail  = get_widget(frmMain, "clistMail");
   gchar *mailbox;
   gchar *mesgid;
   GList *selection;
   guint num = 0, *mesg_nums, i = 0;
   gint y, temp;

   
   gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);

   selection = GTK_CLIST(clistMail)->selection;
   num = g_list_length(selection);
   if (num <= 0)	/* no messages selected */
      return;
      
   mesg_nums = g_malloc0(sizeof(gint) * num);

   selection = GTK_CLIST(clistMail)->selection;
   while(selection != NULL)
   {
      gtk_clist_get_text(GTK_CLIST(clistMail), (gint)selection->data, 0, &mesgid);
      mesg_nums[i] = atoi(mesgid);
      selection = selection->next;
      i++;
   }

   /* gotta sort the ids */
   y = 0;
   while(y == 0)
   {
      y = 1;
      i = 0;
      while(i < num - 1)
      {
         if(mesg_nums[i] > mesg_nums[i+1])
         {
            temp = mesg_nums[i];
            mesg_nums[i] = mesg_nums[i+1];
            mesg_nums[i+1] = temp;
            y = 0;
         }
         i++;
      }
   }

   if (num > 0)
   {
      /* let's really delete that badboy */
      mailbox_del_mesgs(num, mesg_nums, mailbox);
      /*refresh_mail_list(STAY);*/
      clistMail_update();
      mailbox_refresh_mailpos(num, mesg_nums, NumMesgs, mailbox);
      cMesg--;
      if (NumMesgs > 0)
         gtk_clist_select_row(GTK_CLIST(clistMail), cMesg+1, 0);
   }

   if(NumMesgs == 0)
   {
      cMesg = -1;
      refresh_txtMessage();
   }

   g_free(mesg_nums);
}

void on_mnuCopy_to_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   if (!CopyMoveOpen && cMesg >= 0)
	{
      /* show the "Copy To:" dialog */
      cm_operation = MCOPY;
      dlgCopyMove = create_dlgCopyMove("Copy message to...");
      gtk_widget_show(dlgCopyMove);
      CopyMoveOpen = 1;
	}
}

void on_mnuMove_to_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   if (!CopyMoveOpen && cMesg >= 0)
	{
      /* show the "Move To:" dialog */
      cm_operation = MMOVE;
      dlgCopyMove = create_dlgCopyMove("Move message to...");
      gtk_widget_show(dlgCopyMove);
      CopyMoveOpen = 1;
	}
}

void on_mnuMimeSave_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GList *tmp;
	GtkWidget *clistMime = get_widget(frmMain,"clistMime");
	GtkWidget *file_sel;
	gchar name[128];
	gint part,ret;


	tmp = GTK_CLIST(clistMime)->selection;
	if(tmp == NULL)
		return;

	part = (gint)tmp->data;
	tmp = mime_parts;
	while(tmp != NULL && part > 0)
   {
		tmp=tmp->next;
		part--;
	}

	if(((struct mime_part *)tmp->data)->len <= 0)
		return;
		
	file_sel = gtk_file_selection_new(_("Filename"));
	ret = mime_get_parameter_value(((struct mime_part *)tmp->data)->parameter, "name", name);
	if(ret)
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel), name);
		
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->ok_button), "clicked", 
                      GTK_SIGNAL_FUNC(on_mnuMimeSave_file), file_sel);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->cancel_button), "clicked",
                      GTK_SIGNAL_FUNC(on_mnuMimeSave_file_cancel), file_sel);
	gtk_widget_show(file_sel);
}	

void on_mnuMimeSave_file (GtkWidget *button, GtkFileSelection *file_sel)
{
	GtkWidget *clistBoxes = get_widget(frmMain,"clistBoxes");
	GtkWidget *clistMail = get_widget(frmMain,"clistMail");
	GtkWidget *clistMime = get_widget(frmMain,"clistMime");
	gchar *part,*mesg,*mailbox,*mesgid,*file;
	FILE *fp;
	GList *tmp;
	gint len;

	gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);
	
	gtk_clist_get_text(GTK_CLIST(clistMail), cMesg, 0, &mesgid);
	tmp = GTK_CLIST(clistMime)->selection;
	if(tmp == NULL)
		return;

   mesg = mailbox_get_mesg(atoi(mesgid), mailbox);
   /*strip(mesg, '\r');*/
   part = mime_get_part(mesg, ((gint)tmp->data) + 1, &len);
	file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel));
   fp = fopen(file, "wt");
   if (fp != NULL)
   {
      fwrite(part, 1, len, fp);
      fclose(fp);
   }
   else
      fprintf(stderr, _("Couldn't open %s\n"), file);

   g_free(part);
	g_free(mesg);
	
	gtk_widget_destroy(GTK_WIDGET(file_sel));
}

void on_mnuMimeSave_file_cancel (GtkWidget *button, GtkFileSelection *file_sel)
{
   gtk_widget_destroy(GTK_WIDGET(file_sel));
}

void on_mnuAbout_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   if (!AboutOpen)
   {
      /* show the about dialog box... */
      frmAbout = create_frmAbout();
      gtk_widget_show(frmAbout);
      AboutOpen = 1;
   }
}

void on_mnuHowto_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   /* bah, not right now */
}

void on_clistMail_select_row (GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data)
{
   GList *selection;

   selection = GTK_CLIST(clist)->selection;
   /* there's no reason to refresh txtMessage if there are
    * more then one message selected */
   if(g_list_length(selection) == 1 && cMesg != row)
   {
      /* highlight/select the new row */
      cMesg = row;
      refresh_txtMessage();
   }
}

void on_clistMail_clicked (GtkWidget *clist, GdkEvent *event, gpointer user_data)
{
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail  = get_widget(frmMain, "clistMail");
   gchar *mesg = NULL;
   gchar *mailbox = NULL;
   gchar *mesgid;

   if(event->button.button == 1 && (event->type == GDK_2BUTTON_PRESS || 
      event->type == GDK_3BUTTON_PRESS))
   {
      /* okay folks, we got ourselves a double-clicker here... */
      gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);
      gtk_clist_get_text(GTK_CLIST(clistMail), cMesg, 0, &mesgid);

      if (!MesgReaderOpen && cMesg >= 0)
      {
         mesg = mailbox_get_mesg(atoi(mesgid), mailbox);
         frmMesgReader = create_frmMesgReader(mesg, mailbox_get_mesg_subject(mesg));
         gtk_widget_show(frmMesgReader);
         MesgReaderOpen = 1;
         g_free(mesg);
      }
      return;
   }

   if (event->button.button == 3)
   {
      gtk_menu_popup((GtkMenu *)mnuMessage_menu, NULL, NULL, NULL,
                     (gpointer)NULL, (guint)NULL, (guint32)NULL);
   }
}

void on_clistMail_drag_data_get (GtkWidget *clist, GdkDragContext *context, 
                                 GtkSelectionData *selection, guint info, 
                                 guint32 clk_time, gpointer user_data)
{
   char string[32];

   /* the clistboxes widget can get the selected messages' ids itself , no need
    * to send them */
   strcpy(string, "data on the way");
   gtk_selection_data_set(selection, selection->target, 8, string, strlen(string));
}

void on_clistMail_click_column (GtkWidget *clist, gint column, gpointer user_data)
{
   gtk_clist_freeze(GTK_CLIST(clist));

   if (column == sort_column)
   {
      if (sort_type == GTK_SORT_ASCENDING)
         sort_type = GTK_SORT_DESCENDING;
      else
         sort_type = GTK_SORT_ASCENDING;
      gtk_clist_set_sort_type (GTK_CLIST (clist), sort_type);
   }
   else
   {
      sort_column = column;
      gtk_clist_set_sort_column (GTK_CLIST (clist), sort_column);
      sort_type = GTK_SORT_ASCENDING;
      gtk_clist_set_sort_type (GTK_CLIST (clist), sort_type);
      switch (sort_column)
      {
         case 0:         /* sort by message id */
            gtk_clist_set_compare_func (GTK_CLIST (clist), id_compare);
            break;
         case 3:         /* sort by date */
            gtk_clist_set_compare_func (GTK_CLIST (clist), date_compare);
            break;
         default:
            gtk_clist_set_compare_func (GTK_CLIST (clist), str_compare);
      }
   }

   gtk_clist_sort(GTK_CLIST(clist));

   gtk_clist_thaw(GTK_CLIST(clist));
}

void on_clistBoxes_select_row (GtkCList *clist, gint row, gint column,
                               GdkEvent *event, gpointer user_data)
{
   cMbox = row;
   refresh_mail_list(TOP);
}

void on_clistBoxes_clicked (GtkWidget *clist, GdkEvent *event, gpointer user_data)
{
   if(event->button.button == 3)
   {
      gtk_menu_popup((GtkMenu *)mnuMailboxes_menu, NULL, NULL, NULL, 
                     (gpointer)NULL, (guint) NULL, (guint32)NULL);
   }
}

void on_clistBoxes_drag_data_recieved (GtkWidget *clist, GdkDragContext *context, 
                                       gint x, gint y, 
                                       GtkSelectionData *selection_data, 
                                       guint info, guint32 clk_time)
{
   int row, col;
   gchar *mailbox, *mesgid, *oldmailbox;
   GtkWidget *clistMail = get_widget(frmMain,"clistMail");
   GList *selection;
   gint num = 0, *mesg_nums, i = 0, done, temp;
	
   gtk_clist_get_selection_info(GTK_CLIST(clist), x, y, &row, &col);
   gtk_clist_get_text(GTK_CLIST(clist), row, 0, &mailbox);
   gtk_clist_get_text(GTK_CLIST(clist), cMbox, 0, &oldmailbox);

   if (strcmp(mailbox, oldmailbox) == 0)
   {
      return;
   }

   selection = GTK_CLIST(clistMail)->selection;
   while (selection != NULL)
   {
      num++;
      selection = selection->next;
   }

   mesg_nums = g_malloc(sizeof(gint)*num);

   selection = GTK_CLIST(clistMail)->selection;
   while(selection != NULL)
   {
      gtk_clist_get_text(GTK_CLIST(clistMail), (gint)selection->data, 0, 
			 &mesgid);
      mesg_nums[i] = atoi(mesgid);
      i++;
      selection=selection->next;
   }

   done = 0;
   while (done == 0)
   {
      done = 1;
      i = 0;
      while(i < num - 1)
      {
         if(mesg_nums[i] > mesg_nums[i+1])
         {
            temp = mesg_nums[i];
            mesg_nums[i] = mesg_nums[i+1];
            mesg_nums[i+1] = temp;
            done = 0;
         }
         i++;
      }
   }

   if (context->action == GDK_ACTION_MOVE)
   {
      mailbox_move_mesgs(num, mesg_nums, oldmailbox, mailbox);
      mailbox_refresh_mailpos(num, mesg_nums, NumMesgs, oldmailbox);
      clistMail_update();
   }

   if (context->action == GDK_ACTION_COPY)
      mailbox_copy_mesgs(num, mesg_nums, oldmailbox, mailbox);

   g_free(mesg_nums);
   gtk_drag_finish(context, 1, FALSE, clk_time);
}

void on_clistMime_select_row(GtkWidget *clist,gint row,gint col,gpointer user_data)
{
	gint i;
	GList *tmp;

	i=0;
	tmp = mime_parts;
	while(tmp != NULL && i < row)
   {
		tmp=tmp->next;
		i++;
	}
	
	if(!g_strcasecmp(((struct mime_part *)tmp->data)->type,"text") ||
	   !g_strcasecmp(((struct mime_part *)tmp->data)->type,"message"))
   {
		mime_preferred_part=row+1;
		refresh_txtMessage();
	}	
}

void on_frmMainSaveAs_destroy (GtkObject *object, gpointer user_data)
{
   gtk_widget_destroy(frmMainSaveAs);
   SaveAsOpen = 0;
}

void on_cmdMainSaveAs_OK_clicked (GtkButton *button, gpointer user_data)
{
   if (SaveFile != NULL)
      g_free(SaveFile);
   SaveFile = g_malloc(strlen(gtk_file_selection_get_filename(GTK_FILE_SELECTION(frmMainSaveAs))) + 1);
   strcpy(SaveFile, 
	  gtk_file_selection_get_filename(GTK_FILE_SELECTION(frmMainSaveAs)));

   save_message();

   gtk_widget_destroy(frmMainSaveAs);
   SaveAsOpen = 0;
}

void on_cmdMainSaveAs_Cancel_clicked (GtkButton *button, gpointer user_data)
{
   gtk_widget_destroy(frmMainSaveAs);
   SaveAsOpen = 0;
}

void on_dlgNewMailbox_destroy (GtkObject *object, gpointer user_data)
{
   gtk_widget_destroy(dlgNewMailbox);
   MbxNewOpen = 0;
}

void on_cmdDlgNewMailboxOK_clicked (GtkButton *button, gpointer user_data)
{
   GtkWidget *txtNewMailbox = get_widget(dlgNewMailbox, "txtNewMailbox");
   GtkWidget *clist = get_widget(frmMain, "clistBoxes");
   gchar *text[1] = {"Mailbox"};
   gchar *mailbox;

   if ( strlen(gtk_entry_get_text(GTK_ENTRY(txtNewMailbox))) > 0 )
	{
      mailbox = gtk_entry_get_text(GTK_ENTRY(txtNewMailbox));
   
      gtk_clist_freeze(GTK_CLIST(clist));

      gtk_clist_append(GTK_CLIST(clist), (gchar**) text);
      gtk_clist_set_text(GTK_CLIST(clist), NumMailboxes, 0, mailbox);

      mailbox_create(mailbox);

      NumMailboxes++;

      rc_save_defaults();

      gtk_clist_thaw(GTK_CLIST(clist));

      gtk_widget_destroy(dlgNewMailbox);
      MbxNewOpen = 0;

      g_free(mailbox);
	}
}

void on_cmdDlgNewMailboxCancel_clicked (GtkButton *button, gpointer user_data)
{
   gtk_widget_destroy(dlgNewMailbox);
   MbxNewOpen = 0;
}

void on_dlgCopyMove_destroy (GtkObject *object, gpointer user_data)
{
   gtk_widget_destroy(dlgCopyMove);
   CopyMoveOpen = 0;
}

void on_cmdCopyMoveOK_clicked (GtkButton *button, gpointer user_data)
{
   GtkWidget *cmboMailbox = get_widget(dlgCopyMove, "cmboMailbox");
   GtkWidget *clistBoxes  = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail   = get_widget(frmMain, "clistMail");
   gchar *original_mbox;
   gchar *mesgid;
   GList *selection;
   guint num = 0, *mesg_nums, i = 0;
   gint y, temp;

   gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &original_mbox);
   gtk_clist_get_text(GTK_CLIST(clistMail), cMesg, 0, &mesgid);

   selection = GTK_CLIST(clistMail)->selection;
   num = g_list_length(selection);
   if (num <= 0)	/* no messages */
   	return;
   
   mesg_nums = g_malloc0(sizeof(gint) * num);

   selection = GTK_CLIST(clistMail)->selection;
   while (selection != (GList *)NULL)
   {
      gtk_clist_get_text(GTK_CLIST(clistMail), (gint)selection->data, 0, 
			 &mesgid);
      mesg_nums[i] = atoi(mesgid);
      selection = selection->next;
      i++;
   }

   /* gotta sort the ids */
   y = 0;
   while (y == 0)
   {
      y = 1;
      i = 0;
      while (i < num - 1)
      {
         if (mesg_nums[i] > mesg_nums[i+1])
         {
            temp = mesg_nums[i];
            mesg_nums[i] = mesg_nums[i+1];
            mesg_nums[i+1] = temp;
            y = 0;
         }
         i++;
      }
   }

   switch (cm_operation)
	{
      case MCOPY:
         mailbox_copy_mesgs(num, mesg_nums, original_mbox, 
			    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cmboMailbox)->entry)));
         break;
      case MMOVE:
         mailbox_move_mesgs(num, mesg_nums, original_mbox, 
			    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cmboMailbox)->entry)));

         clistMail_update();
         mailbox_refresh_mailpos(num, mesg_nums, NumMesgs, original_mbox);
         cMesg--;
         gtk_clist_select_row(GTK_CLIST(clistMail), cMesg+1, 0);
         /*refresh_mail_list(STAY);*/
         break;
	}

   gtk_widget_destroy(dlgCopyMove);
   g_free(mesg_nums);
   CopyMoveOpen = 0;
}

void on_cmdCopyMoveCancel_clicked (GtkButton *button, gpointer user_data)
{
   gtk_widget_destroy(dlgCopyMove);
   CopyMoveOpen = 0;
}

gint save_message ()
{
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail  = get_widget(frmMain, "clistMail");
   gchar *mailbox;
   gchar *mesg;
   gchar *mesgid;
   FILE *fp;
   gint i;

   gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);
   gtk_clist_get_text(GTK_CLIST(clistMail), cMesg, 0, &mesgid);

   fp = fopen(SaveFile, "wt");
   if (fp == (FILE *)NULL)
      return 0;

   mesg = mailbox_get_mesg(atoi(mesgid), mailbox);
   fprintf(fp, "From:    %s\r\n", mailbox_get_mesg_from(mesg));
   fprintf(fp, "Date:    %s\r\n", mailbox_get_mesg_date(mesg));
   fprintf(fp, "Subject: %s\r\n", mailbox_get_mesg_subject(mesg));
   for (i = 0; i < 78; i++)
      fprintf(fp, "-");
   fprintf(fp, "\r\n\r\n");
   trim_header(mesg, hdr_trim_level);
   fprintf(fp, "%s", mesg);

   fclose(fp);

   g_free(mesg);

   return 1;
}

void refresh_mailboxes (int hilite)
{
   GtkWidget *clist = get_widget(frmMain, "clistBoxes");
   gchar *text[1] = {"Mailbox"};
   FILE *fp;
   gchar buffer[257];

   if (!refreshing_boxes)
   {
      refreshing_boxes = TRUE;

      fp = fopen(spruceconf, "rt");
      if (fp == (FILE *)NULL)
         return;
     
      gtk_clist_freeze(GTK_CLIST(clist));
      gtk_clist_clear(GTK_CLIST(clist));

      NumMailboxes = 0;

      while (!feof(fp))
      {
         memset(buffer, 0, 257);
         fgets(buffer, 256, fp);
         trim_leadspc(buffer);
         if (!strncmp("mailbox", buffer, strlen("mailbox")))
	      {
	         strcut(buffer, 0, strlen("mailbox"));
	         trim_whtspc(buffer);
	         gtk_clist_append(GTK_CLIST(clist), (gchar**) text);
	         gtk_clist_set_text(GTK_CLIST(clist), NumMailboxes, 0, buffer);
	         NumMailboxes++;
	      }
      }
     
      fclose(fp);
     
      if (NumMailboxes)       /* if there are any mailboxes */
      {
         if (hilite == TOP)
      	   cMbox = 0;
         else
	         if (hilite == STAY)
	         {
			      if (cMbox >= NumMailboxes)
				      cMbox = NumMailboxes - 1;
	         }
         gtk_clist_select_row(GTK_CLIST(clist), cMbox, 0);
         gtk_clist_moveto(GTK_CLIST(clist), cMbox, 0, 1.0, 0.0);
         /*refresh_mail_list(TOP);*/
     }
     else
       cMbox = -1;
     
      gtk_clist_thaw(GTK_CLIST(clist));
      refreshing_boxes = FALSE;
   }
}

void refresh_mail_list (gint hilite)
{
   GtkWidget *clist      = get_widget(frmMain, "clistMail");
   GtkWidget *clistBoxes = get_widget(frmMain, "clistBoxes");
   gint i, max;
   gchar *text[4] = {"Id", "Sender", "Subject", "Date"};
   gchar *mesg = NULL;
   gchar *mailbox;
   gchar mesgid[10];

   gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);

   if (!refreshing_maillist)
   {
      refreshing_maillist = TRUE;
      gtk_clist_freeze(GTK_CLIST(clist));
      gtk_clist_clear(GTK_CLIST(clist));

      max = mailbox_get_num_mesgs(mailbox);
      
      for(i = 1; i <= max; i++)
      {
         gtk_clist_append(GTK_CLIST(clist), (gchar**) text);
         mesg = mailbox_get_mesg(i, mailbox);
         if (mesg == (gchar*)NULL)
            break;                         /* this is just a temp fix!! */
         g_snprintf(mesgid, 10, "%d", i);
         gtk_clist_set_text(GTK_CLIST(clist), i-1, 0, mesgid);
         gtk_clist_set_text(GTK_CLIST(clist), i-1, 1, mailbox_get_mesg_from(mesg));
         gtk_clist_set_text(GTK_CLIST(clist), i-1, 2, mailbox_get_mesg_subject(mesg));
         gtk_clist_set_text(GTK_CLIST(clist), i-1, 3, mailbox_get_mesg_date(mesg));
         g_free(mesg);
	   }

      if(max > 0)     /* if there are messages */
	   {
         if (hilite == TOP)
      	   cMesg = 0;
         else
            if (hilite == STAY)
            {
			      if (cMesg >= max)
                  cMesg = max - 1;
			   }
         gtk_clist_sort (GTK_CLIST (clist));  /* sorts the clist to the default column */
         /*refresh_txtMessage();*/
         cMesg--;
         gtk_clist_select_row(GTK_CLIST(clist), cMesg + 1, 0);
         gtk_clist_moveto(GTK_CLIST(clist), cMesg, 0, 1.0, 0.0);
	   }
      else
      {
         cMesg = -1;
         refresh_txtMessage();
      }

		NumMesgs = max;

      /*gtk_clist_sort (GTK_CLIST (clist));*/  /* sorts the clist to the default column */
      gtk_clist_thaw(GTK_CLIST(clist));
      refreshing_maillist = FALSE;
   }
}

void refresh_txtMessage ()
{
   GtkWidget *txtBody      = get_widget(frmMain, "txtMessage");
   GtkWidget *clistBoxes   = get_widget(frmMain, "clistBoxes");
   GtkWidget *clistMail    = get_widget(frmMain, "clistMail");
	GtkWidget *clistMime    = get_widget(frmMain, "clistMime");
   guint len;
   gchar *mesg = NULL;
   gchar *header_mesg = NULL;
   gchar *mailbox;
   gchar *mesgid;
   gchar *part;
   GdkFont *fixed_font = NULL;

   mime = 0;	
   gtk_clist_get_text(GTK_CLIST(clistBoxes), cMbox, 0, &mailbox);

   if (!refreshing_message)
   {
      refreshing_message = 1;
      gtk_text_freeze(GTK_TEXT(txtBody));

      /* delete all the the text in there */
      len = gtk_text_get_length(GTK_TEXT(txtBody));
      gtk_text_set_point(GTK_TEXT(txtBody), len);
      gtk_text_backward_delete(GTK_TEXT(txtBody), len);

      /* and clear the mime clist */
      if (mime_preferred_part == 0)
      	gtk_clist_clear(GTK_CLIST(clistMime));
		
      if (cMesg >= 0)
      {
         /* put the new email in there */
         gtk_clist_get_text(GTK_CLIST(clistMail), cMesg, 0, &mesgid);
         mesg = mailbox_get_mesg(atoi(mesgid), mailbox);
         
         /* since old messages may not have been stripped by pop3io... */
         /*strip(mesg, '\r');*/

         header_mesg = g_strdup(mesg);
         
         if(mime_detect(mesg))
         {
         	mime = 1;
            mime_parse(mesg);
         }
         else
         	trim_header(header_mesg, hdr_trim_level);
         
         
         /* Load a fixed font */
         if (mesg_body_font != (gchar*) NULL)
            fixed_font = gdk_font_load (mesg_body_font);

         if (hdr_trim_level > 0 && mime_preferred_part == 0)
         {
            GdkColormap *cmap;
            GdkColor colour;
            gchar *hdrstripped;
            gint  hdrlen;

            /* Get the system colour map and allocate the colour blue */
            cmap = gdk_colormap_get_system();
            colour.red = 0;
            colour.green = 0;
            colour.blue = 0xffff;
            if (!gdk_color_alloc(cmap, &colour))
               g_error("couldn't allocate colour");

            hdrlen = get_header_len(header_mesg);
            hdrstripped = get_trimmed_header(header_mesg, hdr_trim_level, hdrlen);
            gtk_text_insert (GTK_TEXT (txtBody), fixed_font, &colour, NULL, hdrstripped, -1);
            gtk_text_insert (GTK_TEXT (txtBody), fixed_font, &colour, NULL, "\n\n", -1);
            g_free(hdrstripped);
         }
         
         if (mime)
         {
            if (mime_preferred_part == 0)
               part = mime_get_part(mesg, mime_get_first_text(), &len);
            else
               part = mime_get_part(mesg, mime_preferred_part, &len);

            if (part != (gchar*) NULL)
            {
               gtk_text_insert (GTK_TEXT (txtBody), fixed_font, NULL, NULL, part, -1);
               g_free(part);
            }
            else
            {
               trim_header(mesg, 0);
               gtk_text_insert (GTK_TEXT (txtBody), fixed_font, NULL, NULL, mesg, -1);
            }
            
         }
         else
         {
            trim_header(mesg, 0);
            gtk_text_insert (GTK_TEXT (txtBody), fixed_font, NULL, NULL, mesg, -1);
         }

	      if (mime_preferred_part == 0 && mime)
            refresh_clistMime();
            
         g_free(mesg);
         g_free(header_mesg);
      }

      gtk_text_thaw(GTK_TEXT(txtBody));
      mime_preferred_part = 0;
      refreshing_message = 0;
   }
}

void refresh_clistMime()
{
	GtkWidget *clist = get_widget(frmMain,"clistMime");
	GList *tmp;
	gchar *text[4], id[16], len[32], type[128], name[128];
	gint i, ret;

	gtk_clist_clear(GTK_CLIST(clist));
	
	if(g_list_length(mime_parts) > 1)
   {
		i = 1;
		tmp = mime_parts;
		while(tmp != NULL)
      {
			sprintf(id, "%d", i);
			text[0] = id;
			sprintf(type, "%s/%s", ((struct mime_part *)tmp->data)->type,
					((struct mime_part *)tmp->data)->subtype);
			text[1] = type;
			ret = mime_get_parameter_value(((struct mime_part *)tmp->data)->parameter, "name", name);
			if(ret)
				text[2] = name;
			else
				text[2] = NULL;
			sprintf(len, "%d", ((struct mime_part *)tmp->data)->len);
			text[3] = len;
			gtk_clist_append(GTK_CLIST(clist), text);
			i++;
			tmp = tmp->next;
		}
	}
}

gchar *quote_body (gchar *body, gchar *rcpt, gchar *date)
{
   gchar *new_body;
   gchar *src, *dst, *new_date, *name;
   gint lines = 0;
   gint length;

   /* Count the number of lines in the body. */
   for (src = body; *src; src++)
      if (*src == '\n')
         lines++;

   new_date = (gchar *) g_malloc(strlen(date) + 1);
   dst = new_date;
   /* keep copying over until we hit the first ':' in the time stamp... */
   for (src = date; *src; src++)
   {
      if (*src == ':')
         break;
      *dst = *src;
      dst++;
   }
   /* ...now back-track... */
   for ( ; dst != new_date; dst--)
      if (*dst == ' ')
         break;
   *dst = '\0';

   /* if avail, get the user's name or else just use his email address */
   name = (gchar *) g_malloc0(strlen(rcpt) + 1);
   dst = name;
   for (src = rcpt; *src; src++)
   {
      if (*src == '<' && src != rcpt)
         break;
      *dst = *src;
      dst++;
   }
   *dst = ' ';
   for ( ; dst != name; dst--)
      if (*dst == ' ')
         *dst = '\0';
      else
         break;

   /* Allocate space for the new body. */
   length = (2 * (lines + 1)) + strlen(body) + strlen(name) + strlen(new_date) + 16;
   new_body = (gchar *)g_malloc(length);
   if (new_body == 0)
      return 0;

   /* On $new_date, $recpt wrote: */
   strcpy (new_body, "\n\nOn ");
   strcat (new_body, new_date);
   strcat (new_body, ", ");
   strcat (new_body, name);
   strcat (new_body, " wrote:\n"); 

   /* Reformat the text with leading "> ". */
   dst = new_body + strlen(new_body);
   *dst = '>';
   dst++;
   *dst = ' ';
   dst++;
   for (src = body; *src; src++)
   {
      *dst = *src;
      dst++;
      if (*src == '\n')
      {
         /* Don't quote an empty line. */
         /*if (*(src + 1) == '\n' || *(src + 1) == 0)
            break;*/

         /* Stop quoting at a sig. */
         /*if (*(src + 1) == '-' && *(src + 2) == '-' && *(src + 3) == ' ')
            break;*/

         *dst = '>';
         dst++;
         *dst = ' ';
         dst++;
      }
   }
   *(dst - 2) = '\0';

   return new_body;
}

void spruce_init(int argc, char **argv)
{
   gint cur = 0, *index = 0;

   texticons = 0;

   while (cur != -1)
   {
      cur = getopt_long(argc, argv, "hitbv", CMDLINE, index);

      switch (cur)
      {
         case 'h':
            spruce_help(argv[0]);
            break;
         case 'i':
            if (texticons != TB_BOTH)
               texticons += TB_ICONS;
            break;
         case 't':
            if (texticons != TB_BOTH)
               texticons += TB_TEXT;
            break;
         case 'b':
            texticons = TB_BOTH;
            break;
         case 'v':
            fprintf(stdout, "Spruce v%s\n\n", spruce_ver);
            exit(0);
            break;
         case 1:
            fprintf(stdout, "Key bindings:\n");
            fprintf(stdout, "  Control + C\t\tCopy message...\n");
            fprintf(stdout, "  Control + D\t\tDelete selected message\n");
            fprintf(stdout, "  Control + E\t\tExpunge selected message\n");
            fprintf(stdout, "  Control + F\t\tForward selected message\n");
            fprintf(stdout, "  Control + M\t\tMove message...\n");
            fprintf(stdout, "  Control + N\t\tNew Message\n");
            fprintf(stdout, "  Control + O\t\tOpen selected message\n");
            fprintf(stdout, "  Control + P\t\tPrint Message\n");
            fprintf(stdout, "  Control + R\t\tReply to message\n");
            fprintf(stdout, "  Control + S\t\tSave Message As...\n");
            fprintf(stdout, "  Control + X\t\tExit Spruce\n");
            fprintf(stdout, "  Control + .\t\tNext message\n");
            fprintf(stdout, "  Control + ,\t\tPrevious message\n\n");
            exit(0);
            break;
         default:
            break;
      }
   }

   if (!texticons)
      texticons = TB_BOTH;
}

void spruce_help(gchar *program)
{
  fprintf(stdout, "Usage: %s [options ...]\n", program);
  fprintf(stdout, "Valid options are:\n");
  fprintf(stdout, "  --bindings\t\tShow key bindings\n");
  fprintf(stdout, "  -h, --help \t\tOutput this help\n");
  fprintf(stdout, "  -i, --icons\t\tUse icons in toolbars\n");
  fprintf(stdout, "  -t, --text \t\tUse text in toolbars\n");
  fprintf(stdout, "  -b, --both \t\tUse both text and icons in toolbars\n");
  fprintf(stdout, "  -v, --version\t\tPrint version information\n\n");
  exit(0);
}

void clistMail_update()
{
   GtkWidget *clistMail = get_widget(frmMain, "clistMail");
   gint i, y;
   guint mid, tlid, tmid;   /* message id, temp clist id, temp message id */
   gchar newid[10];
   gchar *mesgid = NULL;
   guint *lids, *mids;     /* clist ids, mesg ids */
   guint clist_size = 0;
   GList *selection;

   /* Lock the list while updating. */
   gtk_clist_freeze(GTK_CLIST(clistMail));

   /* Determine how many items were selected. */
   y = 0;
   selection = GTK_CLIST(clistMail)->selection;
   clist_size = g_list_length(selection);

   lids = (guint *)g_malloc0(sizeof(guint) * clist_size);
   mids = (guint *)g_malloc0(sizeof(guint) * clist_size);

   selection = GTK_CLIST(clistMail)->selection;
   for (y = 0; selection != (GList *)NULL; y++)
   {
      /* Add each id. */
      lids[y] = (guint)selection->data;
      gtk_clist_get_text(GTK_CLIST(clistMail), lids[y], 0, &mesgid);
      mids[y] = atoi(mesgid);

      selection = selection->next;
   }

   /* Sort the ids. */
   for (y = 0; y == 0; y = 1)
   {
      for (i = 0; i < clist_size - 1; i++)
      {
         /* Swap ids if current id is greather than next. */
         if (lids[i] > lids[i+1])
         {
            tlid = lids[i];
            tmid = mids[i];
            lids[i] = lids[i+1];
            mids[i] = mids[i+1];
            lids[i+1] = tlid;
            mids[i+1] = tmid;

	         /* Set flag to continue sorting. */
            y = 0;
         }
      }
   }

   /* Remove the clist items in reverse order, using a temporary copy. */
   for (y = clist_size - 1; y >= 0; y--)
   {
      gtk_clist_remove(GTK_CLIST(clistMail), lids[y]);
      NumMesgs--;
   }

   /* Modify each of the clist items to show their new id numbers, iterating 
      in reverse order to keep the algorithm simple. */
   for (y = clist_size - 1; y >= 0; y--)
   {
      for (i = 0; i < NumMesgs; i++)
      {
         /* Decrement the messageg id of all messages with ids greater than 
            the one we deleted. */
         gtk_clist_get_text(GTK_CLIST(clistMail), i, 0, &mesgid);
         mid = atoi(mesgid);
         if (mid > mids[y])
         {
            /* this message id needs to be decremented */
            g_snprintf(newid, 10, "%d", mid - 1);
            gtk_clist_set_text(GTK_CLIST(clistMail), i, 0, newid);
         }
      }
   }

   cMesg = (cMesg >= NumMesgs ? NumMesgs - 1 : cMesg);
   g_free(lids);
   g_free(mids);

   /* Remove the lock on the compound list. */
   gtk_clist_thaw(GTK_CLIST(clistMail));
}


void read_sizes (void)
{
   GtkWidget *clistMail = get_widget(frmMain, "clistMail");
   FILE *fp;
   gchar buffer[256], *field;
   gint width, heigth, i;

   if (save_sizes)
   {
      fp = fopen(spruceconf, "rt");
      if(fp == NULL)
      {
         fprintf(stderr, "Spruce: %s failed to open for reading.\n", spruceconf);
         fprintf(stderr, "Please check with your local Administrator.\n");
         return;
      }

      while (!feof(fp))
      {
         fgets(buffer, 256, fp);
         trim_leadspc(buffer);
         if (buffer[0] != '#' && buffer[0] != '\n')
         {
            if (!strncmp("spruce-sizes", buffer, strlen("spruce-sizes")))
            {
               strcut(buffer, 0, strlen("spruce-sizes"));
               trim_whtspc(buffer);
               if (!strcmp("",buffer))
                  continue;

               field = get_field(buffer, 1, ':');
               width = atoi(field);
               g_free(field);

               field = get_field(buffer, 2, ':');
               heigth = atoi(field);
               g_free(field);

               gtk_widget_set_usize(frmMain, width, heigth);

               i = 0;
               while (i < GTK_CLIST(clistMail)->columns)
               {
                  field = get_field(buffer, i+3, ':');
                  gtk_clist_set_column_width (GTK_CLIST(clistMail), i, atoi(field));
                  g_free(field);
                  i++;
               }
            }
         }
      }	
   }
}
