/*  Spruce
 *  Copyright (C) 1999 Susixware
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "pgp.h"

static gchar pgp_file[100];
static gchar pgp_file_asc[100];
static gchar chars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

void pgp_gen_filename()
{
   gchar tmp[10];
   guint r, i;
   time_t t;
	
	srand((unsigned) time(&t));

   memset(pgp_file, 0, 99);
   strncpy(pgp_file, "/tmp/spruce", 99);

   memset(tmp, 0, 9);
   for (i = 0; i < 6; i++)
   {
      r = rand() % 61;
      tmp[i] = chars[r];
   }
   strncat(pgp_file, tmp, 99);

   g_snprintf(pgp_file_asc, 99, "%s.asc", pgp_file);
}

gint pgp_detect(gchar *text)
{
   if (strstrcase(text, "-----BEGIN PGP MESSAGE-----"))
      return 1;
   return 0;
}

gchar *pgp_encrypt(gchar *text, gchar *recipients, gchar *passphrase, gint use_gnupg, gint pgp_sign)
{
   gchar pgpgpg[50];
   gchar command[512];
   gchar line[256];
   gchar *buffer;
   FILE *fp;
   /*Execution exec;*/

   pgp_gen_filename();

   /* lets write our mesg to a temp file */
   fp = fopen(pgp_file, "wt");
   if (fp == NULL)
      return NULL;

   fprintf(fp, "%s\r\n", text);
   fflush(fp);

   fclose(fp);

   /* lets generate the command line */
   if (!use_gnupg)
      strcpy(pgpgpg, "pgpe -a");
   else
      strcpy(pgpgpg, "gpg -ea");

   if (pgp_sign)
      strcat(pgpgpg, "s");

   strcat(pgpgpg, "r");

   g_snprintf(command, 512, "%s %s %s", pgpgpg, recipients, pgp_file);

   fp = popen(command, "wt");
   if (fp == NULL)
      return NULL;

   fprintf(fp, "%s\n", passphrase);  /* lets send our passphrase... */
   fflush(fp);

   /* pgp should now have encrypted our mesg */
   pclose(fp);

   /* now lets read the encrypted message in */
   fp = fopen(pgp_file_asc, "rt");
   if (fp == NULL)
   {
      remove(pgp_file);
      return NULL;
   }

   buffer = g_malloc(1);
   memset(buffer, 0, 1);
   while (!feof(fp))
   {
      memset(line, 0, 255);
      fgets(line, 256, fp);
      buffer = g_realloc(buffer, strlen(buffer) + strlen(line) + 1);
      strcat(buffer, line);
   }

   fclose(fp);
   remove(pgp_file);
   remove(pgp_file_asc);

   return buffer;
}

gchar *pgp_decrypt(gchar *text, gchar *passphrase, gint use_gnupg)
{
   gchar crypt_bin[50];   /* Binary used for PGP. */
   gchar command[512];    /* Command used to execute binary. */
   gchar line[256];
   gchar *buffer;
   FILE *fp;

   pgp_gen_filename();

   /* lets write our mesg to a temp file */
   fp = fopen(pgp_file_asc, "wt");
   if (fp == NULL)
      return NULL;

   fprintf(fp, "%s\r\n", text);
   fflush(fp);

   fclose(fp);

   /* Generate the command line. */
   if (!use_gnupg)
      strcpy(crypt_bin, "pgpv");
   else
      strcpy(crypt_bin, "gpg -d");

   g_snprintf(command, 512, "%s %s", crypt_bin, pgp_file_asc);

   if (use_gnupg)
   {
      strcat(command, " > ");
      strcat(command, pgp_file);
   }

   fp = popen(command, "wt");
   if (fp == NULL)
      return NULL;

   fprintf(fp, "%s\n", passphrase);  /* lets send our passphrase... */
   fflush(fp);

   /* pgp should now have decrypted our mesg */
   pclose(fp);

   fp = fopen(pgp_file_asc, "rt");
   if (fp == NULL)
   {
      remove(pgp_file);
      return NULL;
   }

   buffer = g_malloc(2);
   memset(buffer, 0, 1);
   while (!feof(fp))
   {
      memset(line, 0, 255);
      fgets(line, 256, fp);
      buffer = g_realloc(buffer, strlen(buffer) + strlen(line) + 1);
      strcat(buffer, line);
   }

   fclose(fp);
   remove(pgp_file);
   remove(pgp_file_asc);

   return buffer;
}
